@@ -13,11 +13,12 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
+#include "core.h"
 #include "module.h"
 #include "modules.h"
 #include "signals.h"
@@ -46,15 +47,19 @@ typedef struct {
         int signal_args[MAX_EXPANDO_SIGNALS];
 } EXPANDO_REC;
 
+const char *current_expando = NULL;
+
 static int timer_tag;
 
 static EXPANDO_REC *char_expandos[255];
 static GHashTable *expandos;
-static time_t client_start_time;
 static char *last_sent_msg, *last_sent_msg_body;
 static char *last_privmsg_from, *last_public_from;
 static char *sysname, *sysrelease, *sysarch;
-static const char *timestamp_format;
+
+static char *timestamp_format;
+static int timestamp_seconds;
+static time_t last_timestamp;
 
 #define CHAR_EXPANDO(chr) \
 	(char_expandos[(int) (unsigned char) chr])
@@ -66,7 +71,7 @@ void expando_create(const char *key, EXPANDO_FUNC func, ...)
         const char *signal;
 	va_list va;
 
-	g_return_if_fail(key != NULL || *key == '\0');
+	g_return_if_fail(key != NULL && *key != '\0');
 	g_return_if_fail(func != NULL);
 
 	if (key[1] != '\0')
@@ -128,10 +133,10 @@ void expando_add_signal(const char *key, const char *signal, ExpandoArg arg)
 /* Destroy expando */
 void expando_destroy(const char *key, EXPANDO_FUNC func)
 {
-	gpointer origkey;
+	gpointer origkey, value;
         EXPANDO_REC *rec;
 
-	g_return_if_fail(key != NULL || *key == '\0');
+	g_return_if_fail(key != NULL && *key != '\0');
 	g_return_if_fail(func != NULL);
 
 	if (key[1] == '\0') {
@@ -141,8 +146,9 @@ void expando_destroy(const char *key, EXPANDO_FUNC func)
 			char_expandos[(int) (unsigned char) *key] = NULL;
 			g_free(rec);
 		}
-	} else if (g_hash_table_lookup_extended(expandos, key, &origkey,
-						(gpointer *) &rec)) {
+	} else if (g_hash_table_lookup_extended(expandos, key,
+						&origkey, &value)) {
+		rec = value;
 		if (rec->func == func) {
 			g_hash_table_remove(expandos, key);
 			g_free(origkey);
@@ -176,7 +182,8 @@ void expando_bind(const char *key, int funccount, SIGNAL_FUNC *funcs)
 		func = arg < funccount ? funcs[arg] : NULL;
 		if (func == NULL) func = funcs[EXPANDO_ARG_NONE];
 
-		signal_add_to_id(MODULE_NAME, 1, rec->signal_ids[n], func);
+		signal_add_full_id(MODULE_NAME, SIGNAL_PRIORITY_DEFAULT,
+				   rec->signal_ids[n], func, NULL);
 	}
 }
 
@@ -205,10 +212,42 @@ void expando_unbind(const char *key, int funccount, SIGNAL_FUNC *funcs)
 		func = arg < funccount ? funcs[arg] : NULL;
 		if (func == NULL) func = funcs[EXPANDO_ARG_NONE];
 
-		signal_remove_id(rec->signal_ids[n], func);
+		signal_remove_id(rec->signal_ids[n], func, NULL);
 	}
 }
 
+/* Returns [<signal id>, EXPANDO_ARG_xxx, <signal id>, ..., -1] */
+int *expando_get_signals(const char *key)
+{
+	EXPANDO_REC *rec;
+	int *signals;
+        int n;
+
+	g_return_val_if_fail(key != NULL, NULL);
+
+	rec = expando_find(key);
+	if (rec == NULL || rec->signals < 0)
+                return NULL;
+
+	if (rec->signals == 0) {
+		/* it's unknown when this expando changes..
+		   check it once in a second */
+		signals = g_new(int, 3);
+		signals[0] = signal_get_uniq_id("expando timer");
+		signals[1] = EXPANDO_ARG_NONE;
+		signals[2] = -1;
+                return signals;
+	}
+
+        signals = g_new(int, rec->signals*2+1);
+	for (n = 0; n < rec->signals; n++) {
+                signals[n*2] = rec->signal_ids[n];
+                signals[n*2+1] = rec->signal_args[n];
+	}
+	signals[rec->signals*2] = -1;
+        return signals;
+}
+
 EXPANDO_FUNC expando_find_char(char chr)
 {
 	return CHAR_EXPANDO(chr) == NULL ? NULL :
@@ -221,6 +260,13 @@ EXPANDO_FUNC expando_find_long(const char *key)
 	return rec == NULL ? NULL : rec->func;
 }
 
+static gboolean free_expando(gpointer key, gpointer value, gpointer user_data)
+{
+	g_free(key);
+	g_free(value);
+	return TRUE;
+}
+
 /* last person who sent you a MSG */
 static char *expando_lastmsg(SERVER_REC *server, void *item, int *free_ret)
 {
@@ -273,7 +319,7 @@ static char *expando_last_invite(SERVER_REC *server, void *item, int *free_ret)
 /* client version text string */
 static char *expando_version(SERVER_REC *server, void *item, int *free_ret)
 {
-	return IRSSI_VERSION;
+	return PACKAGE_VERSION;
 }
 
 /* current value of CMDCHARS */
@@ -282,10 +328,38 @@ static char *expando_cmdchars(SERVER_REC *server, void *item, int *free_ret)
 	return (char *) settings_get_str("cmdchars");
 }
 
+/* first CMDCHAR */
+static char *expando_cmdchar(SERVER_REC *server, void *item, int *free_ret)
+{
+	char str[2] = { 0, 0 };
+
+	str[0] = *settings_get_str("cmdchars");
+
+	*free_ret = TRUE;
+	return g_strdup(str);
+}
+
 /* modes of current channel, if any */
 static char *expando_chanmode(SERVER_REC *server, void *item, int *free_ret)
 {
-	return !IS_CHANNEL(item) ? NULL : CHANNEL(item)->mode;
+	char *cmode;
+	char *args;
+
+	*free_ret = FALSE;
+
+	if (!IS_CHANNEL(item))
+		return NULL;
+
+        if (!settings_get_bool("chanmode_expando_strip"))
+		return CHANNEL(item)->mode;
+
+	*free_ret = TRUE;
+	cmode = g_strdup(CHANNEL(item)->mode);
+	args = strchr(cmode, ' ');
+	if (args != NULL)
+		*args = 0;
+
+	return cmode;
 }
 
 /* current nickname */
@@ -322,13 +396,22 @@ static char *expando_serverversion(SERVER_REC *server, void *item, int *free_ret
 /* target of current input (channel or QUERY nickname) */
 static char *expando_target(SERVER_REC *server, void *item, int *free_ret)
 {
-	return item == NULL ? "" : ((WI_ITEM_REC *) item)->name;
+	return item == NULL ? "" :
+		(char *) window_item_get_target((WI_ITEM_REC *) item);
 }
 
-/* client release date (numeric version string) */
+/* client release date (in YYYYMMDD format) */
 static char *expando_releasedate(SERVER_REC *server, void *item, int *free_ret)
 {
-	return IRSSI_VERSION_DATE;
+        *free_ret = TRUE;
+	return g_strdup_printf("%d", IRSSI_VERSION_DATE);
+}
+
+/* client release time (in HHMM format) */
+static char *expando_releasetime(SERVER_REC *server, void *item, int *free_ret)
+{
+        *free_ret = TRUE;
+	return g_strdup_printf("%04d", IRSSI_VERSION_TIME);
 }
 
 /* current working directory */
@@ -388,8 +471,21 @@ static char *expando_sysarch(SERVER_REC *server, void *item, int *free_ret)
 /* Topic of active channel (or address of queried nick) */
 static char *expando_topic(SERVER_REC *server, void *item, int *free_ret)
 {
-	return IS_CHANNEL(item) ? CHANNEL(item)->topic :
-		IS_QUERY(item) ? QUERY(item)->address : "";
+	if (IS_CHANNEL(item))
+		return CHANNEL(item)->topic;
+	if (IS_QUERY(item)) {
+		QUERY_REC *query = QUERY(item);
+
+		if (query->server_tag == NULL)
+			return "";
+
+                *free_ret = TRUE;
+		return query->address == NULL ?
+			g_strdup_printf("(%s)", query->server_tag) :
+			g_strdup_printf("%s (%s)", query->address,
+					query->server_tag);
+	}
+        return "";
 }
 
 /* Server tag */
@@ -404,6 +500,12 @@ static char *expando_chatnet(SERVER_REC *server, void *item, int *free_ret)
 	return server == NULL ? "" : server->connrec->chatnet;
 }
 
+/* visible_name of current window item */
+static char *expando_itemname(SERVER_REC *server, void *item, int *free_ret)
+{
+	return item == NULL ? "" : ((WI_ITEM_REC *) item)->visible_name;
+}
+
 static void sig_message_public(SERVER_REC *server, const char *msg,
 			       const char *nick, const char *address,
 			       const char *target)
@@ -437,13 +539,44 @@ static void sig_message_own_private(SERVER_REC *server, const char *msg,
 
 static int sig_timer(void)
 {
+	time_t now;
+	struct tm *tm;
+        int last_min;
+
         signal_emit("expando timer", 0);
+
+        /* check if $Z has changed */
+	now = time(NULL);
+	if (last_timestamp != now) {
+		if (!timestamp_seconds && last_timestamp != 0) {
+                        /* assume it changes every minute */
+			tm = localtime(&last_timestamp);
+			last_min = tm->tm_min;
+
+			tm = localtime(&now);
+			if (tm->tm_min == last_min)
+                                return 1;
+		}
+
+                signal_emit("time changed", 0);
+		last_timestamp = now;
+	}
+
         return 1;
 }
 
 static void read_settings(void)
 {
-        timestamp_format = settings_get_str("timestamp_format");
+	g_free_not_null(timestamp_format);
+	timestamp_format = g_strdup(settings_get_str("timestamp_format"));
+
+	timestamp_seconds =
+		strstr(timestamp_format, "%r") != NULL ||
+		strstr(timestamp_format, "%s") != NULL ||
+		strstr(timestamp_format, "%S") != NULL ||
+		strstr(timestamp_format, "%X") != NULL ||
+		strstr(timestamp_format, "%T") != NULL;
+
 }
 
 void expandos_init(void)
@@ -452,11 +585,12 @@ void expandos_init(void)
 	struct utsname un;
 #endif
 	settings_add_str("misc", "STATUS_OPER", "*");
-	settings_add_str("misc", "timestamp_format", "%H:%M");
+	settings_add_str("lookandfeel", "timestamp_format", "%H:%M");
+	settings_add_bool("lookandfeel", "chanmode_expando_strip", FALSE);
 
-	client_start_time = time(NULL);
 	last_sent_msg = NULL; last_sent_msg_body = NULL;
 	last_privmsg_from = NULL; last_public_from = NULL;
+        last_timestamp = 0;
 
         sysname = sysrelease = sysarch = NULL;
 #ifdef HAVE_SYS_UTSNAME_H
@@ -491,12 +625,15 @@ void expandos_init(void)
 		       "", EXPANDO_NEVER, NULL);
 	expando_create("K", expando_cmdchars,
 		       "setup changed", EXPANDO_ARG_NONE, NULL);
+	expando_create("k", expando_cmdchar,
+		       "setup changed", EXPANDO_ARG_NONE, NULL);
 	expando_create("M", expando_chanmode,
 		       "window changed", EXPANDO_ARG_NONE,
 		       "window item changed", EXPANDO_ARG_WINDOW,
 		       "channel mode changed", EXPANDO_ARG_WINDOW_ITEM, NULL);
 	expando_create("N", expando_nick,
 		       "window changed", EXPANDO_ARG_NONE,
+		       "window connect changed", EXPANDO_ARG_WINDOW,
 		       "window server changed", EXPANDO_ARG_WINDOW,
                        "server nick changed", EXPANDO_ARG_SERVER, NULL);
 	expando_create("O", expando_statusoper,
@@ -519,11 +656,15 @@ void expandos_init(void)
 		       "window item changed", EXPANDO_ARG_WINDOW, NULL);
 	expando_create("V", expando_releasedate,
 		       "", EXPANDO_NEVER, NULL);
+	expando_create("versiontime", expando_releasetime,
+		       "", EXPANDO_NEVER, NULL);
 	expando_create("W", expando_workdir, NULL);
 	expando_create("Y", expando_realname,
 		       "window changed", EXPANDO_ARG_NONE,
+		       "window connect changed", EXPANDO_ARG_WINDOW,
 		       "window server changed", EXPANDO_ARG_WINDOW, NULL);
-	expando_create("Z", expando_time, NULL);
+	expando_create("Z", expando_time,
+		       "time changed", EXPANDO_ARG_NONE, NULL);
 	expando_create("$", expando_dollar,
 		       "", EXPANDO_NEVER, NULL);
 
@@ -540,10 +681,17 @@ void expandos_init(void)
 		       "query address changed", EXPANDO_ARG_WINDOW_ITEM, NULL);
 	expando_create("tag", expando_servertag,
 		       "window changed", EXPANDO_ARG_NONE,
+		       "window connect changed", EXPANDO_ARG_WINDOW,
 		       "window server changed", EXPANDO_ARG_WINDOW, NULL);
 	expando_create("chatnet", expando_chatnet,
 		       "window changed", EXPANDO_ARG_NONE,
+		       "window connect changed", EXPANDO_ARG_WINDOW,
 		       "window server changed", EXPANDO_ARG_WINDOW, NULL);
+	expando_create("itemname", expando_itemname,
+		       "window changed", EXPANDO_ARG_NONE,
+		       "window item changed", EXPANDO_ARG_WINDOW,
+		       "window item name changed", EXPANDO_ARG_WINDOW_ITEM,
+		       NULL);
 
 	read_settings();
 
@@ -561,21 +709,19 @@ void expandos_deinit(void)
 	for (n = 0; n < sizeof(char_expandos)/sizeof(char_expandos[0]); n++)
 		g_free_not_null(char_expandos[n]);
 
-	expando_destroy("sysname", expando_sysname);
-	expando_destroy("sysrelease", expando_sysrelease);
-	expando_destroy("sysarch", expando_sysarch);
-	expando_destroy("topic", expando_topic);
-	expando_destroy("tag", expando_servertag);
-	expando_destroy("chatnet", expando_chatnet);
+	g_hash_table_foreach_remove(expandos, free_expando, NULL);
+	g_hash_table_destroy(expandos);
 
-        g_hash_table_destroy(expandos);
+	g_free_not_null(last_sent_msg);
+	g_free_not_null(last_sent_msg_body);
+	g_free_not_null(last_privmsg_from);
+	g_free_not_null(last_public_from);
+	g_free_not_null(sysname);
+	g_free_not_null(sysrelease);
+	g_free_not_null(sysarch);
+	g_free_not_null(timestamp_format);
 
-	g_free_not_null(last_sent_msg); g_free_not_null(last_sent_msg_body);
-	g_free_not_null(last_privmsg_from); g_free_not_null(last_public_from);
-	g_free_not_null(sysname); g_free_not_null(sysrelease);
-        g_free_not_null(sysarch);
-
-        g_source_remove(timer_tag);
+	g_source_remove(timer_tag);
 	signal_remove("message public", (SIGNAL_FUNC) sig_message_public);
 	signal_remove("message private", (SIGNAL_FUNC) sig_message_private);
 	signal_remove("message own_private", (SIGNAL_FUNC) sig_message_own_private);
