@@ -58,25 +58,6 @@ static int ignore_check_replies_rec(IGNORE_REC *rec, CHANNEL_REC *channel,
 	return FALSE;
 }
 
-static int ignore_check_replies(CHANNEL_REC *chanrec, const char *text)
-{
-	GSList *tmp;
-
-	if (text == NULL || chanrec == NULL)
-		return FALSE;
-
-        /* check reply ignores */
-	for (tmp = ignores; tmp != NULL; tmp = tmp->next) {
-		IGNORE_REC *rec = tmp->data;
-
-		if (rec->mask != NULL && rec->replies &&
-		    ignore_check_replies_rec(rec, chanrec, text))
-			return TRUE;
-	}
-
-	return FALSE;
-}
-
 static int ignore_match_pattern(IGNORE_REC *rec, const char *text)
 {
 	if (rec->pattern == NULL)
@@ -99,8 +80,15 @@ static int ignore_match_pattern(IGNORE_REC *rec, const char *text)
 		stristr(text, rec->pattern) != NULL;
 }
 
+/* MSGLEVEL_NO_ACT is special in ignores, when provided to ignore_check() it's
+ * used as a flag to indicate it should only look at ignore items with NO_ACT.
+ * However we also want to allow NO_ACT combined with levels, so mask it out and
+ * match levels if set. */
 #define ignore_match_level(rec, level) \
-        ((level & (rec)->level) != 0)
+        (((level & MSGLEVEL_NO_ACT) != 0) ? \
+         ((~MSGLEVEL_NO_ACT & level) & (rec)->level) != 0 : \
+         ((rec)->level & MSGLEVEL_NO_ACT ? 0 : \
+         (level & (rec)->level) != 0))
 
 #define ignore_match_nickmask(rec, nick, nickmask) \
 	((rec)->mask == NULL || \
@@ -110,42 +98,31 @@ static int ignore_match_pattern(IGNORE_REC *rec, const char *text)
 
 #define ignore_match_server(rec, server) \
 	((rec)->servertag == NULL || \
-	g_strcasecmp((server)->tag, (rec)->servertag) == 0)
+	g_ascii_strcasecmp((server)->tag, (rec)->servertag) == 0)
 
 #define ignore_match_channel(rec, channel) \
 	((rec)->channels == NULL || ((channel) != NULL && \
 		strarray_find((rec)->channels, (channel)) != -1))
 
-static int ignore_check_without_mask(GSList *list, CHANNEL_REC *channel,
-				     int level, const char *text)
+static int ignore_check_replies(CHANNEL_REC *chanrec, const char *text, int level)
 {
 	GSList *tmp;
-        int len, best_mask, best_match, best_patt;
 
-        best_mask = best_patt = -1; best_match = FALSE;
-	for (tmp = list; tmp != NULL; tmp = tmp->next) {
+	if (text == NULL || chanrec == NULL)
+		return FALSE;
+
+        /* check reply ignores */
+	for (tmp = ignores; tmp != NULL; tmp = tmp->next) {
 		IGNORE_REC *rec = tmp->data;
 
-		if (ignore_match_level(rec, level) &&
-		    ignore_match_pattern(rec, text)) {
-			len = rec->mask == NULL ? 0 : strlen(rec->mask);
-			if (len > best_mask) {
-				best_mask = len;
-				best_match = !rec->exception;
-			} else if (len == best_mask && rec->pattern != NULL) {
-				len = strlen(rec->pattern);
-				if (len > best_patt) {
-					best_patt = len;
-					best_match = !rec->exception;
-				}
-			}
-		}
+		if (rec->mask != NULL && rec->replies &&
+		    ignore_match_level(rec, level) &&
+		    ignore_match_channel(rec, chanrec->name) &&
+		    ignore_check_replies_rec(rec, chanrec, text))
+			return TRUE;
 	}
 
-	if (best_match || (level & MSGLEVEL_PUBLIC) == 0)
-		return best_match;
-
-        return ignore_check_replies(channel, text);
+	return FALSE;
 }
 
 int ignore_check(SERVER_REC *server, const char *nick, const char *host,
@@ -154,7 +131,7 @@ int ignore_check(SERVER_REC *server, const char *nick, const char *host,
 	CHANNEL_REC *chanrec;
 	NICK_REC *nickrec;
         IGNORE_REC *rec;
-	GSList *tmp, *list;
+	GSList *tmp;
         char *nickmask;
         int len, best_mask, best_match, best_patt;
 
@@ -169,31 +146,36 @@ int ignore_check(SERVER_REC *server, const char *nick, const char *host,
 		if (nickrec->host == NULL)
 			nicklist_set_host(chanrec, nickrec, host);
 
-		list = nickmatch_find(nickmatch, nickrec);
-		return ignore_check_without_mask(list, chanrec, level, text);
+		tmp = nickmatch_find(nickmatch, nickrec);
+		nickmask = NULL;
+	} else {
+		tmp = ignores;
+		nickmask = g_strconcat(nick, "!", host, NULL);
 	}
 
-	nickmask = g_strconcat(nick, "!", host, NULL);
-
         best_mask = best_patt = -1; best_match = FALSE;
-	for (tmp = ignores; tmp != NULL; tmp = tmp->next) {
+	for (; tmp != NULL; tmp = tmp->next) {
+		int match = 1;
 		rec = tmp->data;
 
-		if (ignore_match_level(rec, level) &&
-		    ignore_match_server(rec, server) &&
-		    ignore_match_channel(rec, channel) &&
-		    ignore_match_nickmask(rec, nick, nickmask) &&
+		if (nickmask != NULL)
+			match = ignore_match_server(rec, server) &&
+				ignore_match_channel(rec, channel) &&
+				ignore_match_nickmask(rec, nick, nickmask);
+		if (match &&
+		    ignore_match_level(rec, level) &&
 		    ignore_match_pattern(rec, text)) {
 			len = rec->mask == NULL ? 0 : strlen(rec->mask);
 			if (len > best_mask) {
 				best_mask = len;
 				best_match = !rec->exception;
-			} else if (len == best_mask && rec->pattern != NULL) {
-				len = strlen(rec->pattern);
+			} else if (len == best_mask) {
+				len = rec->pattern == NULL ? 0 : strlen(rec->pattern);
 				if (len > best_patt) {
 					best_patt = len;
 					best_match = !rec->exception;
-				}
+				} else if (len == best_patt && rec->exception)
+					best_match = 0;
 			}
 		}
 	}
@@ -202,11 +184,18 @@ int ignore_check(SERVER_REC *server, const char *nick, const char *host,
 	if (best_match || (level & MSGLEVEL_PUBLIC) == 0)
 		return best_match;
 
-        return ignore_check_replies(chanrec, text);
+        return ignore_check_replies(chanrec, text, level);
 }
 
 IGNORE_REC *ignore_find(const char *servertag, const char *mask,
-			char **channels)
+		char **channels)
+{
+	return ignore_find_noact(servertag, mask, channels, 0);
+}
+
+
+IGNORE_REC *ignore_find_noact(const char *servertag, const char *mask,
+		char **channels, int noact)
 {
 	GSList *tmp;
 	char **chan;
@@ -224,14 +213,20 @@ IGNORE_REC *ignore_find(const char *servertag, const char *mask,
 			    (servertag != NULL && rec->servertag == NULL))
 				continue;
 
-			if (servertag != NULL && g_strcasecmp(servertag, rec->servertag) != 0)
+			if (servertag != NULL && g_ascii_strcasecmp(servertag, rec->servertag) != 0)
 				continue;
 		}
 
+		if (noact && (rec->level & MSGLEVEL_NO_ACT) == 0)
+			continue;
+
+		if (!noact && (rec->level & MSGLEVEL_NO_ACT) != 0)
+			continue;
+
 		if ((rec->mask == NULL && mask != NULL) ||
 		    (rec->mask != NULL && mask == NULL)) continue;
 
-		if (rec->mask != NULL && g_strcasecmp(rec->mask, mask) != 0)
+		if (rec->mask != NULL && g_ascii_strcasecmp(rec->mask, mask) != 0)
 			continue;
 
 		if ((channels == NULL && rec->channels == NULL))
@@ -319,10 +314,24 @@ static void ignore_remove_config(IGNORE_REC *rec)
 static void ignore_init_rec(IGNORE_REC *rec)
 {
 #ifdef HAVE_REGEX_H
+	char *errbuf;
+	int errcode, errbuf_len;
+
 	if (rec->regexp_compiled) regfree(&rec->preg);
-	rec->regexp_compiled = !rec->regexp || rec->pattern == NULL ? FALSE :
-		regcomp(&rec->preg, rec->pattern,
-			REG_EXTENDED|REG_ICASE|REG_NOSUB) == 0;
+	rec->regexp_compiled = FALSE;
+	if (rec->regexp && rec->pattern != NULL) {
+		errcode = regcomp(&rec->preg, rec->pattern,
+				REG_EXTENDED|REG_ICASE|REG_NOSUB);
+		if (errcode != 0) {
+			errbuf_len = regerror(errcode, &rec->preg, 0, 0);
+			errbuf = g_malloc(errbuf_len);
+			regerror(errcode, &rec->preg, errbuf, errbuf_len);
+			g_warning("Failed to compile regexp '%s': %s", rec->pattern, errbuf);
+			g_free(errbuf);
+		} else {
+			rec->regexp_compiled = TRUE;
+		}
+	}
 #endif
 }
 
@@ -421,7 +430,7 @@ static void read_ignores(void)
 
 		rec->mask = g_strdup(config_node_get_str(node, "mask", NULL));
 		rec->pattern = g_strdup(config_node_get_str(node, "pattern", NULL));
-		rec->level = level2bits(config_node_get_str(node, "level", ""));
+		rec->level = level2bits(config_node_get_str(node, "level", ""), NULL);
                 rec->exception = config_node_get_bool(node, "exception", FALSE);
 		rec->regexp = config_node_get_bool(node, "regexp", FALSE);
 		rec->fullword = config_node_get_bool(node, "fullword", FALSE);
