@@ -46,6 +46,7 @@ static const char *levels[] = {
 	"HILIGHTS",
 
 	"NOHILIGHT",
+	"NO_ACT",
 	NULL
 };
 
@@ -53,19 +54,22 @@ int level_get(const char *level)
 {
 	int n, len, match;
 
-	if (g_strcasecmp(level, "ALL") == 0 || strcmp(level, "*") == 0)
+	if (g_ascii_strcasecmp(level, "ALL") == 0 || strcmp(level, "*") == 0)
 		return MSGLEVEL_ALL;
 
-	if (g_strcasecmp(level, "NEVER") == 0)
+	if (g_ascii_strcasecmp(level, "NEVER") == 0)
 		return MSGLEVEL_NEVER;
 
+	if (g_ascii_strcasecmp(level, "NO_ACT") == 0)
+		return MSGLEVEL_NO_ACT;
+
 	len = strlen(level);
 	if (len == 0) return 0;
 
 	/* partial match allowed, as long as it's the only one that matches */
 	match = 0;
 	for (n = 0; levels[n] != NULL; n++) {
-		if (g_strncasecmp(levels[n], level, len) == 0) {
+		if (g_ascii_strncasecmp(levels[n], level, len) == 0) {
 			if ((int)strlen(levels[n]) == len) {
 				/* full match */
 				return 1L << n;
@@ -81,11 +85,14 @@ int level_get(const char *level)
 	return match;
 }
 
-int level2bits(const char *level)
+int level2bits(const char *level, int *errorp)
 {
 	char *orig, *str, *ptr;
 	int ret, singlelevel, negative;
 
+	if (errorp != NULL)
+		*errorp = FALSE;
+
 	g_return_val_if_fail(level != NULL, 0);
 
 	if (*level == '\0')
@@ -107,7 +114,8 @@ int level2bits(const char *level)
 		if (singlelevel != 0) {
 			ret = !negative ? (ret | singlelevel) :
 				(ret & ~singlelevel);
-		}
+		} else if (errorp != NULL)
+			*errorp = TRUE;
 
        		while (*str == ' ') str++;
 		if (*str == '\0') break;
@@ -128,16 +136,25 @@ char *bits2level(int bits)
 	if (bits == 0)
 		return g_strdup("");
 
-	if (bits == MSGLEVEL_ALL)
-		return g_strdup("ALL");
 
 	str = g_string_new(NULL);
-	if (bits & MSGLEVEL_NEVER)
+	if (bits & MSGLEVEL_NEVER) {
 		g_string_append(str, "NEVER ");
+		bits &= ~MSGLEVEL_NEVER;
+	}
 
-	for (n = 0; levels[n] != NULL; n++) {
-		if (bits & (1L << n))
-			g_string_sprintfa(str, "%s ", levels[n]);
+	if (bits & MSGLEVEL_NO_ACT) {
+		g_string_append(str, "NO_ACT ");
+		bits &= ~MSGLEVEL_NO_ACT;
+	}
+
+	if (bits == MSGLEVEL_ALL) {
+		g_string_append(str, "ALL ");
+	} else {
+		for (n = 0; levels[n] != NULL; n++) {
+			if (bits & (1L << n))
+				g_string_append_printf(str, "%s ", levels[n]);
+		}
 	}
         if (str->len > 0)
 		g_string_truncate(str, str->len-1);
@@ -158,7 +175,6 @@ int combine_level(int dest, const char *src)
 	list = g_strsplit(src, " ", -1);
 	for (item = list; *item != NULL; item++) {
 		itemname = *item + (**item == '+' || **item == '-' ? 1 : 0);
-                g_strup(itemname);
 		itemlevel = level_get(itemname);
 
 		if (strcmp(itemname, "NONE") == 0)
