@@ -140,7 +140,7 @@ int find_substr(const char *list, const char *item)
 		ptr = strchr(list, ' ');
 		if (ptr == NULL) ptr = list+strlen(list);
 
-		if (g_strncasecmp(list, item, ptr-list) == 0 &&
+		if (g_ascii_strncasecmp(list, item, ptr-list) == 0 &&
 		    item[ptr-list] == '\0')
 			return TRUE;
 
@@ -174,7 +174,7 @@ int strarray_find(char **array, const char *item)
 
 	index = 0;
 	for (tmp = array; *tmp != NULL; tmp++, index++) {
-		if (g_strcasecmp(*tmp, item) == 0)
+		if (g_ascii_strcasecmp(*tmp, item) == 0)
 			return index;
 	}
 
@@ -183,7 +183,7 @@ int strarray_find(char **array, const char *item)
 
 GSList *gslist_find_string(GSList *list, const char *key)
 {
-	for (list = list; list != NULL; list = list->next)
+	for (; list != NULL; list = list->next)
 		if (strcmp(list->data, key) == 0) return list;
 
 	return NULL;
@@ -191,8 +191,8 @@ GSList *gslist_find_string(GSList *list, const char *key)
 
 GSList *gslist_find_icase_string(GSList *list, const char *key)
 {
-	for (list = list; list != NULL; list = list->next)
-		if (g_strcasecmp(list->data, key) == 0) return list;
+	for (; list != NULL; list = list->next)
+		if (g_ascii_strcasecmp(list->data, key) == 0) return list;
 
 	return NULL;
 }
@@ -268,7 +268,7 @@ GSList *hashtable_get_keys(GHashTable *hash)
 
 GList *glist_find_string(GList *list, const char *key)
 {
-	for (list = list; list != NULL; list = list->next)
+	for (; list != NULL; list = list->next)
 		if (strcmp(list->data, key) == 0) return list;
 
 	return NULL;
@@ -276,8 +276,8 @@ GList *glist_find_string(GList *list, const char *key)
 
 GList *glist_find_icase_string(GList *list, const char *key)
 {
-	for (list = list; list != NULL; list = list->next)
-		if (g_strcasecmp(list->data, key) == 0) return list;
+	for (; list != NULL; list = list->next)
+		if (g_ascii_strcasecmp(list->data, key) == 0) return list;
 
 	return NULL;
 }
@@ -443,12 +443,12 @@ char *convert_home(const char *path)
 
 int g_istr_equal(gconstpointer v, gconstpointer v2)
 {
-	return g_strcasecmp((const char *) v, (const char *) v2) == 0;
+	return g_ascii_strcasecmp((const char *) v, (const char *) v2) == 0;
 }
 
 int g_istr_cmp(gconstpointer v, gconstpointer v2)
 {
-	return g_strcasecmp((const char *) v, (const char *) v2);
+	return g_ascii_strcasecmp((const char *) v, (const char *) v2);
 }
 
 /* a char* hash function from ASU */
@@ -966,3 +966,21 @@ char *ascii_strdown(char *str)
 		*s = g_ascii_tolower (*s);
 	return str;
 }
+
+char **strsplit_len(const char *str, int len)
+{
+	char **ret;
+	size_t total_len = strlen(str);
+	int n = total_len / len;
+	int i;
+
+	if (total_len % len)
+		n++;
+
+	ret = g_new(char *, n + 1);
+	for (i = 0; i < n; i++, str += len)
+		ret[i] = g_strndup(str, len);
+	ret[n] = NULL;
+
+	return ret;
+}
