@@ -477,7 +477,7 @@ static GIOChannel *irssi_ssl_get_iochannel(GIOChannel *handle, int port, SERVER_
 	}
 	SSL_CTX_set_options(ctx, SSL_OP_NO_SSLv2);
 	SSL_CTX_set_default_passwd_cb(ctx, get_pem_password_callback);
-	SSL_CTX_set_default_passwd_cb_userdata(ctx, mypass);
+	SSL_CTX_set_default_passwd_cb_userdata(ctx, (void *)mypass);
 
 	if (mycert && *mycert) {
 		char *scert = NULL, *spkey = NULL;
