@@ -60,7 +60,11 @@ GIOChannel *g_io_channel_new(int handle)
 
 IPADDR ip4_any = {
 	AF_INET,
+#if defined(HAVE_IPV6) && defined(IN6ADDR_ANY_INIT)
+	IN6ADDR_ANY_INIT
+#else
 	{ INADDR_ANY }
+#endif
 };
 
 int net_ip_compare(IPADDR *ip1, IPADDR *ip2)
@@ -260,7 +264,7 @@ void net_disconnect(GIOChannel *handle)
 {
 	g_return_if_fail(handle != NULL);
 
-	g_io_channel_close(handle);
+	g_io_channel_shutdown(handle, TRUE, NULL);
 	g_io_channel_unref(handle);
 }
 
