@@ -22,6 +22,7 @@
 #include "signals.h"
 #include "commands.h"
 #include "args.h"
+#include "network.h"
 #include "net-sendbuffer.h"
 #include "pidwait.h"
 #include "lib-config/iconfig.h"
@@ -91,17 +92,14 @@ static void cmd_upgrade(const char *data)
 static void session_save_nick(CHANNEL_REC *channel, NICK_REC *nick,
 			      CONFIG_REC *config, CONFIG_NODE *node)
 {
-	static char other[2];
 	node = config_node_section(node, NULL, NODE_TYPE_BLOCK);
 
 	config_node_set_str(config, node, "nick", nick->nick);
 	config_node_set_bool(config, node, "op", nick->op);
 	config_node_set_bool(config, node, "halfop", nick->halfop);
 	config_node_set_bool(config, node, "voice", nick->voice);
-	
-	other[0] = nick->other;
-	other[1] = '\0';
-	config_node_set_str(config, node, "other", other);
+
+	config_node_set_str(config, node, "prefixes", nick->prefixes);
 
 	signal_emit("session save nick", 4, channel, nick, config, node);
 }
@@ -262,10 +260,10 @@ static void session_restore_server(CONFIG_NODE *node)
 				  chatnet, password, nick);
 	if (conn != NULL) {
 		conn->reconnection = TRUE;
-		conn->connect_handle = g_io_channel_unix_new(handle);
+		conn->connect_handle = g_io_channel_new(handle);
 
 		server = proto->server_init_connect(conn);
-		server->version = g_strdup(config_node_get_str(node, "version", NULL));		
+		server->version = g_strdup(config_node_get_str(node, "version", NULL));
 		server->session_reconnect = TRUE;
 		signal_emit("session restore server", 2, server, node);
 
@@ -287,7 +285,7 @@ static void sig_session_save(CONFIG_REC *config)
 	/* save pids */
         str = g_string_new(NULL);
 	for (tmp = pidwait_get_pids(); tmp != NULL; tmp = tmp->next)
-                g_string_sprintfa(str, "%d ", GPOINTER_TO_INT(tmp->data));
+                g_string_append_printf(str, "%d ", GPOINTER_TO_INT(tmp->data));
         config_node_set_str(config, config->mainnode, "pids", str->str);
         g_string_free(str, TRUE);
 }
@@ -329,14 +327,13 @@ static void sig_init_finished(void)
 	config_close(session);
 
 	unlink(session_file);
-	session_file = NULL;
 }
 
 void session_register_options(void)
 {
-	static struct poptOption options[] = {
-		{ "session", 0, POPT_ARG_STRING | POPT_ARGFLAG_DOC_HIDDEN, &session_file, 0, "Used by /UPGRADE command", "PATH" },
-		{ NULL, '\0', 0, NULL }
+	static GOptionEntry options[] = {
+		{ "session", 0, G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_STRING, &session_file, "Used by /UPGRADE command", "PATH" },
+		{ NULL }
 	};
 
 	session_file = NULL;
