@@ -45,16 +45,19 @@
 
 GHashTable *printnicks;
 
-/* convert _underlined_ and *bold* words (and phrases) to use real
+/* convert _underlined_, /italics/, and *bold* words (and phrases) to use real
    underlining or bolding */
 char *expand_emphasis(WI_ITEM_REC *item, const char *text)
 {
 	GString *str;
 	char *ret;
 	int pos;
+	int emphasis_italics;
 
         g_return_val_if_fail(text != NULL, NULL);
 
+	emphasis_italics = settings_get_bool("emphasis_italics");
+
 	str = g_string_new(text);
 
 	for (pos = 0; pos < str->len; pos++) {
@@ -62,9 +65,11 @@ char *expand_emphasis(WI_ITEM_REC *item, const char *text)
 
 		bgn = str->str + pos;
 
-		if (*bgn == '*') 
+		if (*bgn == '*')
 			type = 2; /* bold */
-		else if (*bgn == '_') 
+		else if (*bgn == '/' && emphasis_italics)
+			type = 29; /* italics */
+		else if (*bgn == '_')
 			type = 31; /* underlined */
 		else
 			continue;
@@ -76,7 +81,10 @@ char *expand_emphasis(WI_ITEM_REC *item, const char *text)
 		if ((end = strchr(bgn+1, *bgn)) == NULL)
 			continue;
 		if (!ishighalnum(end[-1]) || ishighalnum(end[1]) ||
-		    end[1] == type || end[1] == '*' || end[1] == '_')
+		    end[1] == type || end[1] == '*' || end[1] == '_' ||
+		    /* special case for italics to not emphasise
+		       common paths by skipping /.../.X */
+		    (type == 29 && i_ispunct(end[1]) && ishighalnum(end[2])))
 			continue;
 
 		if (IS_CHANNEL(item)) {
@@ -84,7 +92,7 @@ char *expand_emphasis(WI_ITEM_REC *item, const char *text)
 			   use emphasis on them. */
 			int found;
                         char c;
-			char *end2; 
+			char *end2;
 
 			/* check if _foo_ is a nick */
 			c = end[1];
@@ -92,7 +100,7 @@ char *expand_emphasis(WI_ITEM_REC *item, const char *text)
                         found = nicklist_find(CHANNEL(item), bgn) != NULL;
 			end[1] = c;
 			if (found) continue;
-			
+
 			/* check if the whole 'word' (e.g. "_foo_^") is a nick
 			   in "_foo_^ ", end will be the second _, end2 the ^ */
 			end2 = end;
@@ -190,6 +198,9 @@ static void sig_message_public(SERVER_REC *server, const char *msg,
 	if (for_me)
 		level |= MSGLEVEL_HILIGHT;
 
+	if (ignore_check(server, nick, address, target, msg, level | MSGLEVEL_NO_ACT))
+		level |= MSGLEVEL_NO_ACT;
+
 	if (settings_get_bool("emphasis"))
 		msg = freemsg = expand_emphasis((WI_ITEM_REC *) chanrec, msg);
 
@@ -223,7 +234,7 @@ static void sig_message_public(SERVER_REC *server, const char *msg,
 				    for_me ? TXT_PUBMSG_ME_CHANNEL :
 				    TXT_PUBMSG_CHANNEL,
 				    printnick, target, msg, nickmode);
-	}						
+	}
 
 	g_free_not_null(nickmode);
 	g_free_not_null(freemsg);
@@ -235,13 +246,17 @@ static void sig_message_private(SERVER_REC *server, const char *msg,
 {
 	QUERY_REC *query;
         char *freemsg = NULL;
+	int level = MSGLEVEL_MSGS;
 
 	query = query_find(server, nick);
 
 	if (settings_get_bool("emphasis"))
 		msg = freemsg = expand_emphasis((WI_ITEM_REC *) query, msg);
 
-	printformat(server, nick, MSGLEVEL_MSGS,
+	if (ignore_check(server, nick, address, NULL, msg, level | MSGLEVEL_NO_ACT))
+		level |= MSGLEVEL_NO_ACT;
+
+	printformat(server, nick, level,
 		    query == NULL ? TXT_MSG_PRIVATE :
 		    TXT_MSG_PRIVATE_QUERY, nick, address, msg);
 
@@ -325,7 +340,12 @@ static void sig_message_own_private(SERVER_REC *server, const char *msg,
 static void sig_message_join(SERVER_REC *server, const char *channel,
 			     const char *nick, const char *address)
 {
-	printformat(server, channel, MSGLEVEL_JOINS,
+	int level = MSGLEVEL_JOINS;
+
+	if (ignore_check(server, nick, address, channel, NULL, level | MSGLEVEL_NO_ACT))
+		level |= MSGLEVEL_NO_ACT;
+
+	printformat(server, channel, level,
 		    TXT_JOIN, nick, address, channel);
 }
 
@@ -333,7 +353,12 @@ static void sig_message_part(SERVER_REC *server, const char *channel,
 			     const char *nick, const char *address,
 			     const char *reason)
 {
-	printformat(server, channel, MSGLEVEL_PARTS,
+	int level = MSGLEVEL_PARTS;
+
+	if (ignore_check(server, nick, address, channel, NULL, level | MSGLEVEL_NO_ACT))
+		level |= MSGLEVEL_NO_ACT;
+
+	printformat(server, channel, level,
 		    TXT_PART, nick, address, channel, reason);
 }
 
@@ -344,17 +369,21 @@ static void sig_message_quit(SERVER_REC *server, const char *nick,
 	GString *chans;
 	GSList *tmp, *windows;
 	char *print_channel;
-	int once, count;
+	int once, count, level = MSGLEVEL_QUITS;
 
 	if (ignore_check(server, nick, address, NULL, reason, MSGLEVEL_QUITS))
 		return;
 
+	if (ignore_check(server, nick, address, NULL, reason, level | MSGLEVEL_NO_ACT))
+		level |= MSGLEVEL_NO_ACT;
+
 	print_channel = NULL;
 	once = settings_get_bool("show_quit_once");
 
 	count = 0; windows = NULL;
 	chans = g_string_new(NULL);
 	for (tmp = server->channels; tmp != NULL; tmp = tmp->next) {
+		level = MSGLEVEL_QUITS;
 		CHANNEL_REC *rec = tmp->data;
 
 		if (!nicklist_find(rec, nick))
@@ -366,6 +395,9 @@ static void sig_message_quit(SERVER_REC *server, const char *nick,
 			continue;
 		}
 
+		if (ignore_check(server, nick, address, rec->visible_name, reason, MSGLEVEL_NO_ACT))
+			level |= MSGLEVEL_NO_ACT;
+
 		if (print_channel == NULL ||
 		    active_win->active == (WI_ITEM_REC *) rec)
 			print_channel = rec->visible_name;
@@ -377,7 +409,7 @@ static void sig_message_quit(SERVER_REC *server, const char *nick,
 			if (g_slist_find(windows, window) == NULL) {
 				windows = g_slist_append(windows, window);
 				printformat(server, rec->visible_name,
-					    MSGLEVEL_QUITS,
+					    level,
 					    TXT_QUIT, nick, address, reason,
 					    rec->visible_name);
 			}
@@ -391,7 +423,7 @@ static void sig_message_quit(SERVER_REC *server, const char *nick,
 		   display the quit there too */
 		QUERY_REC *query = query_find(server, nick);
 		if (query != NULL) {
-			printformat(server, nick, MSGLEVEL_QUITS,
+			printformat(server, nick, level,
 				    TXT_QUIT, nick, address, reason, "");
 		}
 	}
@@ -410,7 +442,12 @@ static void sig_message_kick(SERVER_REC *server, const char *channel,
 			     const char *nick, const char *kicker,
 			     const char *address, const char *reason)
 {
-	printformat(server, channel, MSGLEVEL_KICKS,
+	int level = MSGLEVEL_KICKS;
+
+	if (ignore_check(server, kicker, address, channel, reason, level | MSGLEVEL_NO_ACT))
+		level |= MSGLEVEL_NO_ACT;
+
+	printformat(server, channel, level,
 		    TXT_KICK, nick, channel, kicker, reason, address);
 }
 
@@ -428,6 +465,9 @@ static void print_nick_change_channel(SERVER_REC *server, const char *channel,
 	level = MSGLEVEL_NICKS;
         if (ownnick) level |= MSGLEVEL_NO_ACT;
 
+	if (!(level & MSGLEVEL_NO_ACT) && ignore_check(server, oldnick, address, channel, newnick, level | MSGLEVEL_NO_ACT))
+		level |= MSGLEVEL_NO_ACT;
+
 	printformat(server, channel, level,
 		    ownnick ? TXT_YOUR_NICK_CHANGED : TXT_NICK_CHANGED,
 		    oldnick, newnick, channel, address);
@@ -502,7 +542,12 @@ static void sig_message_topic(SERVER_REC *server, const char *channel,
 			      const char *topic,
 			      const char *nick, const char *address)
 {
-	printformat(server, channel, MSGLEVEL_TOPICS,
+	int level = MSGLEVEL_TOPICS;
+
+	if (ignore_check(server, nick, address, channel, topic, level | MSGLEVEL_NO_ACT))
+		level |= MSGLEVEL_NO_ACT;
+
+	printformat(server, channel, level,
 		    *topic != '\0' ? TXT_NEW_TOPIC : TXT_TOPIC_UNSET,
 		    nick, channel, topic, address);
 }
@@ -643,6 +688,7 @@ void fe_messages_init(void)
 	settings_add_bool("lookandfeel", "emphasis", TRUE);
 	settings_add_bool("lookandfeel", "emphasis_replace", FALSE);
 	settings_add_bool("lookandfeel", "emphasis_multiword", FALSE);
+	settings_add_bool("lookandfeel", "emphasis_italics", FALSE);
 	settings_add_bool("lookandfeel", "show_nickmode", TRUE);
 	settings_add_bool("lookandfeel", "show_nickmode_empty", TRUE);
 	settings_add_bool("lookandfeel", "print_active_channel", FALSE);
