@@ -13,9 +13,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
 #include "module.h"
@@ -40,7 +40,7 @@ static NICKMATCH_REC *nickmatch;
 static int never_hilight_level, default_hilight_level;
 GSList *hilights;
 
-static void reset_cache(void)
+static void reset_level_cache(void)
 {
 	GSList *tmp;
 
@@ -51,7 +51,11 @@ static void reset_cache(void)
 		if (never_hilight_level & rec->level)
                         never_hilight_level &= ~rec->level;
 	}
+}
 
+static void reset_cache(void)
+{
+	reset_level_cache();
 	nickmatch_rebuild(nickmatch);
 }
 
@@ -74,6 +78,7 @@ static void hilight_add_config(HILIGHT_REC *rec)
         if (rec->nickmask) iconfig_node_set_bool(node, "mask", TRUE);
         if (rec->fullword) iconfig_node_set_bool(node, "fullword", TRUE);
         if (rec->regexp) iconfig_node_set_bool(node, "regexp", TRUE);
+        if (rec->servertag) iconfig_node_set_str(node, "servertag", rec->servertag);
 
 	if (rec->channels != NULL && *rec->channels != NULL) {
 		node = config_node_section(node, "channels", NODE_TYPE_LIST);
@@ -157,7 +162,7 @@ static HILIGHT_REC *hilight_find(const char *text, char **channels)
 	for (tmp = hilights; tmp != NULL; tmp = tmp->next) {
 		HILIGHT_REC *rec = tmp->data;
 
-		if (g_strcasecmp(rec->text, text) != 0)
+		if (g_ascii_strcasecmp(rec->text, text) != 0)
 			continue;
 
 		if ((channels == NULL && rec->channels == NULL))
@@ -263,6 +268,8 @@ HILIGHT_REC *hilight_match(SERVER_REC *server, const char *channel,
 
 		if (!rec->nickmask && hilight_match_level(rec, level) &&
 		    hilight_match_channel(rec, channel) &&
+		    (rec->servertag == NULL ||
+		     (server != NULL && g_ascii_strcasecmp(rec->servertag, server->tag) == 0)) &&
 		    hilight_match_text(rec, str, match_beg, match_end))
 			return rec;
 	}
@@ -332,6 +339,12 @@ static void sig_print_text(TEXT_DEST_REC *dest, const char *text,
 	if (!nick_match || (dest->level & MSGLEVEL_HILIGHT)) {
 		/* update the level / hilight info */
 		hilight_update_text_dest(dest, hilight);
+		/* Remove NO_ACT, this means explicitly defined hilights will bypass
+		 * /IGNORE ... NO_ACT.
+		 * (It's still possible to use /hilight -actcolor %n to hide
+		 * hilight/beep).
+		 */
+		dest->level &= ~MSGLEVEL_NO_ACT;
 	}
 
 	if (nick_match)
@@ -353,7 +366,7 @@ static void sig_print_text(TEXT_DEST_REC *dest, const char *text,
 	} else {
 		/* hilight part of the line */
                 GString *tmp;
-                char *middle, *lastcolor;
+                char *middle;
 		int pos, color_pos, color_len;
 
                 tmp = g_string_new(NULL);
@@ -377,16 +390,13 @@ static void sig_print_text(TEXT_DEST_REC *dest, const char *text,
 		pos = strip_real_length(text, hilight_end,
 					&color_pos, &color_len);
 		if (color_pos > 0)
-			lastcolor = g_strndup(text+color_pos, color_len);
+			g_string_append_len(tmp, text+color_pos, color_len);
                 else {
                         /* no colors in line, change back to default */
-			lastcolor = g_malloc0(3);
-			lastcolor[0] = 4;
-                        lastcolor[1] = FORMAT_STYLE_DEFAULTS;
+			g_string_append_c(tmp, 4);
+			g_string_append_c(tmp, FORMAT_STYLE_DEFAULTS);
 		}
-		g_string_append(tmp, lastcolor);
 		g_string_append(tmp, text+pos);
-		g_free(lastcolor);
 
                 newstr = tmp->str;
                 g_string_free(tmp, FALSE);
@@ -458,7 +468,7 @@ static void read_hilight_config(void)
 		rec->nickmask = config_node_get_bool(node, "mask", FALSE);
 		rec->fullword = config_node_get_bool(node, "fullword", FALSE);
 		rec->regexp = config_node_get_bool(node, "regexp", FALSE);
-
+		rec->servertag = config_node_get_str(node, "servertag", NULL);
 		hilight_init_rec(rec);
 
 		node = config_node_section(node, "channels", -1);
@@ -479,8 +489,8 @@ static void hilight_print(int index, HILIGHT_REC *rec)
 		if (rec->word) g_string_append(options, "-word ");
 	}
 
-	if (rec->nickmask) g_string_append(options, "-nickmask ");
-	if (rec->fullword) g_string_append(options, "-fullword ");
+	if (rec->nickmask) g_string_append(options, "-mask ");
+	if (rec->fullword) g_string_append(options, "-full ");
 	if (rec->regexp) {
 		g_string_append(options, "-regexp ");
 #ifdef HAVE_REGEX_H
@@ -490,11 +500,13 @@ static void hilight_print(int index, HILIGHT_REC *rec)
 	}
 
 	if (rec->priority != 0)
-		g_string_sprintfa(options, "-priority %d ", rec->priority);
+		g_string_append_printf(options, "-priority %d ", rec->priority);
+	if (rec->servertag != NULL)
+		g_string_append_printf(options, "-network %s ", rec->servertag);
 	if (rec->color != NULL)
-		g_string_sprintfa(options, "-color %s ", rec->color);
+		g_string_append_printf(options, "-color %s ", rec->color);
 	if (rec->act_color != NULL)
-		g_string_sprintfa(options, "-actcolor %s ", rec->act_color);
+		g_string_append_printf(options, "-actcolor %s ", rec->act_color);
 
 	chans = rec->channels == NULL ? NULL :
 		g_strjoinv(",", rec->channels);
@@ -529,12 +541,12 @@ static void cmd_hilight_show(void)
 
 /* SYNTAX: HILIGHT [-nick | -word | -line] [-mask | -full | -regexp]
                    [-color <color>] [-actcolor <color>] [-level <level>]
-		   [-channels <channels>] <text> */
+		   [-network <network>] [-channels <channels>] <text> */
 static void cmd_hilight(const char *data)
 {
         GHashTable *optlist;
 	HILIGHT_REC *rec;
-	char *colorarg, *actcolorarg, *levelarg, *priorityarg, *chanarg, *text;
+	char *colorarg, *actcolorarg, *levelarg, *priorityarg, *chanarg, *text, *servertag;
 	char **channels;
 	void *free_arg;
 
@@ -554,11 +566,12 @@ static void cmd_hilight(const char *data)
 	priorityarg = g_hash_table_lookup(optlist, "priority");
 	colorarg = g_hash_table_lookup(optlist, "color");
 	actcolorarg = g_hash_table_lookup(optlist, "actcolor");
+	servertag = g_hash_table_lookup(optlist, "network");
 
 	if (*text == '\0') cmd_param_error(CMDERR_NOT_ENOUGH_PARAMS);
 
 	channels = (chanarg == NULL || *chanarg == '\0') ? NULL :
-		g_strsplit(replace_chars(chanarg, ',', ' '), " ", -1);
+		g_strsplit(chanarg, ",", -1);
 
 	rec = hilight_find(text, channels);
 	if (rec == NULL) {
@@ -575,7 +588,7 @@ static void cmd_hilight(const char *data)
 	}
 
 	rec->level = (levelarg == NULL || *levelarg == '\0') ? 0 :
-		level2bits(replace_chars(levelarg, ',', ' '));
+		level2bits(replace_chars(levelarg, ',', ' '), NULL);
 	rec->priority = priorityarg == NULL ? 0 : atoi(priorityarg);
 
 	if (g_hash_table_lookup(optlist, "line") != NULL) {
@@ -605,6 +618,11 @@ static void cmd_hilight(const char *data)
 		if (*actcolorarg != '\0')
 			rec->act_color = g_strdup(actcolorarg);
 	}
+	if (servertag != NULL) {
+		g_free_and_null(rec->servertag);
+		if (*servertag != '\0')
+			rec->servertag = g_strdup(servertag);
+	}
 
 	hilight_create(rec);
 
@@ -675,6 +693,7 @@ static void hilight_nick_cache(GHashTable *list, CHANNEL_REC *channel,
 static void read_settings(void)
 {
 	default_hilight_level = settings_get_level("hilight_level");
+	reset_level_cache();
 }
 
 void hilight_text_init(void)
@@ -694,7 +713,7 @@ void hilight_text_init(void)
 
 	command_bind("hilight", NULL, (SIGNAL_FUNC) cmd_hilight);
 	command_bind("dehilight", NULL, (SIGNAL_FUNC) cmd_dehilight);
-	command_set_options("hilight", "-color -actcolor -level -priority -channels nick word line mask full regexp");
+	command_set_options("hilight", "-color -actcolor -level -priority -network -channels nick word line mask full regexp");
 }
 
 void hilight_text_deinit(void)
