@@ -164,7 +164,7 @@ KEYINFO_REC *key_info_find(const char *id)
 	for (tmp = keyinfos; tmp != NULL; tmp = tmp->next) {
 		KEYINFO_REC *rec = tmp->data;
 
-		if (g_strcasecmp(rec->id, id) == 0)
+		if (g_ascii_strcasecmp(rec->id, id) == 0)
 			return rec;
 	}
 
@@ -304,6 +304,20 @@ static int expand_key(const char *key, GSList **out)
                         /* possibly beginning of keycombo */
 			start = key;
                         last_hyphen = FALSE;
+		} else if (g_utf8_validate(key, -1, NULL)) {
+			/* Assume we are looking at the start of a
+			 * multibyte sequence we will receive as-is,
+			 * so add it to the list as-is.
+			 */
+			const char *p, *end = g_utf8_next_char(key);
+			for (p = key; p != end; p++)
+				expand_out_char(*out, *p);
+			expand_out_char(*out, '-');
+			/* The for loop skips past the remaining character.
+			 * Nasty, I know...
+			 */
+			key = end - 1;
+			last_hyphen = FALSE;
 		} else {
 			expand_out_char(*out, *key);
 			expand_out_char(*out, '-');
@@ -360,8 +374,8 @@ static void key_states_rescan(void)
 
 	memset(used_keys, 0, sizeof(used_keys));
 
-	g_tree_traverse(key_states, (GTraverseFunc) key_state_destroy,
-			G_IN_ORDER, NULL);
+	g_tree_foreach(key_states, (GTraverseFunc) key_state_destroy,
+			NULL);
 	g_tree_destroy(key_states);
 	key_states = g_tree_new((GCompareFunc) strcmp);
 
@@ -661,7 +675,7 @@ static void cmd_show_keys(const char *searchkey, int full)
 		for (key = rec->keys; key != NULL; key = key->next) {
 			KEY_REC *rec = key->data;
 
-			if ((len == 0 || g_strncasecmp(rec->key, searchkey, len) == 0) &&
+			if ((len == 0 || g_ascii_strncasecmp(rec->key, searchkey, len) == 0) &&
 			    (!full || rec->key[len] == '\0')) {
 				printformat(NULL, NULL, MSGLEVEL_CLIENTCRAP, TXT_BIND_LIST,
 					    rec->key, rec->info->id, rec->data == NULL ? "" : rec->data);
@@ -739,7 +753,7 @@ static GList *completion_get_keyinfos(const char *info)
 	for (tmp = keyinfos; tmp != NULL; tmp = tmp->next) {
 		KEYINFO_REC *rec = tmp->data;
 
-		if (g_strncasecmp(rec->id, info, len) == 0)
+		if (g_ascii_strncasecmp(rec->id, info, len) == 0)
                         list = g_list_append(list, g_strdup(rec->id));
 	}
 
@@ -870,8 +884,8 @@ void keyboard_deinit(void)
 	g_hash_table_destroy(keys);
 	g_hash_table_destroy(default_keys);
 
-	g_tree_traverse(key_states, (GTraverseFunc) key_state_destroy,
-			G_IN_ORDER, NULL);
+	g_tree_foreach(key_states, (GTraverseFunc) key_state_destroy,
+			NULL);
 	g_tree_destroy(key_states);
 
 	signal_remove("irssi init read settings", (SIGNAL_FUNC) read_keyboard_config);
