@@ -13,9 +13,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
 #include "module.h"
@@ -34,34 +34,38 @@
 
 static int beep_msg_level, beep_when_away, beep_when_window_active;
 
-static int signal_gui_print_text;
-static int signal_print_text_stripped;
-static int signal_print_text;
-static int signal_print_text_finished;
-static int signal_print_format;
+static int signal_gui_print_text_finished;
 static int signal_print_starting;
+static int signal_print_text;
+static int signal_print_format;
 
 static int sending_print_starting;
 
 static void print_line(TEXT_DEST_REC *dest, const char *text);
 
-static void printformat_module_dest(const char *module, TEXT_DEST_REC *dest,
-				    int formatnum, va_list va)
+void printformat_module_dest_args(const char *module, TEXT_DEST_REC *dest,
+				  int formatnum, va_list va)
 {
 	char *arglist[MAX_FORMAT_PARAMS];
 	char buffer[DEFAULT_FORMAT_ARGLIST_SIZE];
 	FORMAT_REC *formats;
-	THEME_REC *theme;
-	char *str;
-
-	theme = dest->window->theme == NULL ? current_theme :
-		dest->window->theme;
 
 	formats = g_hash_table_lookup(default_formats, module);
 	format_read_arglist(va, &formats[formatnum],
 			    arglist, sizeof(arglist)/sizeof(char *),
 			    buffer, sizeof(buffer));
 
+	printformat_module_dest_charargs(module, dest, formatnum, arglist);
+}
+
+void printformat_module_dest_charargs(const char *module, TEXT_DEST_REC *dest,
+				      int formatnum, char **arglist)
+{
+	THEME_REC *theme;
+	char *str;
+
+	theme = window_get_theme(dest->window);
+
 	if (!sending_print_starting) {
 		sending_print_starting = TRUE;
 		signal_emit_id(signal_print_starting, 1, dest);
@@ -73,10 +77,20 @@ static void printformat_module_dest(const char *module, TEXT_DEST_REC *dest,
 
 	str = format_get_text_theme_charargs(theme, module, dest,
 					     formatnum, arglist);
-	if (*str != '\0') print_line(dest, str);
+	if (str != NULL && *str != '\0') print_line(dest, str);
 	g_free(str);
 }
 
+void printformat_module_dest(const char *module, TEXT_DEST_REC *dest,
+			     int formatnum, ...)
+{
+	va_list va;
+
+	va_start(va, formatnum);
+	printformat_module_dest_args(module, dest, formatnum, va);
+	va_end(va);
+}
+
 void printformat_module_args(const char *module, void *server,
 			     const char *target, int level,
 			     int formatnum, va_list va)
@@ -84,7 +98,7 @@ void printformat_module_args(const char *module, void *server,
 	TEXT_DEST_REC dest;
 
 	format_create_dest(&dest, server, target, level, NULL);
-	printformat_module_dest(module, &dest, formatnum, va);
+	printformat_module_dest_args(module, &dest, formatnum, va);
 }
 
 void printformat_module(const char *module, void *server, const char *target,
@@ -103,7 +117,7 @@ void printformat_module_window_args(const char *module, WINDOW_REC *window,
 	TEXT_DEST_REC dest;
 
 	format_create_dest(&dest, NULL, NULL, level, window);
-	printformat_module_dest(module, &dest, formatnum, va);
+	printformat_module_dest_args(module, &dest, formatnum, va);
 }
 
 void printformat_module_window(const char *module, WINDOW_REC *window,
@@ -133,7 +147,8 @@ void printformat_module_gui_args(const char *module, int formatnum, va_list va)
 			    arglist, sizeof(arglist)/sizeof(char *),
 			    buffer, sizeof(buffer));
 
-	str = format_get_text_theme_charargs(current_theme, module, &dest,
+	str = format_get_text_theme_charargs(window_get_theme(dest.window),
+					     module, &dest,
 					     formatnum, arglist);
 	if (*str != '\0') format_send_to_gui(&dest, str);
 	g_free(str);
@@ -150,22 +165,25 @@ void printformat_module_gui(const char *module, int formatnum, ...)
 
 static void print_line(TEXT_DEST_REC *dest, const char *text)
 {
-	char *str, *tmp;
+        THEME_REC *theme;
+	char *str, *tmp, *stripped;
 
 	g_return_if_fail(dest != NULL);
 	g_return_if_fail(text != NULL);
 
-	tmp = format_get_level_tag(current_theme, dest);
-	str = format_add_linestart(text, tmp);
+        theme = window_get_theme(dest->window);
+	tmp = format_get_level_tag(theme, dest);
+	str = !theme->info_eol ? format_add_linestart(text, tmp) :
+		format_add_lineend(text, tmp);
 	g_free_not_null(tmp);
 
-	/* send the plain text version for logging etc.. */
-	tmp = strip_codes(str);
-	signal_emit_id(signal_print_text_stripped, 2, dest, tmp);
-	g_free(tmp);
+	/* send both the formatted + stripped (for logging etc.) */
+	stripped = strip_codes(str);
+	signal_emit_id(signal_print_text, 3, dest, str, stripped);
+        g_free_and_null(dest->hilight_color);
 
-	signal_emit_id(signal_print_text, 2, dest, str);
 	g_free(str);
+        g_free(stripped);
 }
 
 /* append string to `out', expand newlines. */
@@ -188,6 +206,7 @@ static char *printtext_get_args(TEXT_DEST_REC *dest, const char *str,
 {
 	GString *out;
 	char *ret;
+	int adv;
 
 	out = g_string_new(NULL);
 	for (; *str != '\0'; str++) {
@@ -208,38 +227,41 @@ static char *printtext_get_args(TEXT_DEST_REC *dest, const char *str,
 		}
 		case 'd': {
 			int d = (int) va_arg(va, int);
-			g_string_sprintfa(out, "%d", d);
+			g_string_append_printf(out, "%d", d);
 			break;
 		}
 		case 'f': {
 			double f = (double) va_arg(va, double);
-			g_string_sprintfa(out, "%0.2f", f);
+			g_string_append_printf(out, "%0.2f", f);
 			break;
 		}
 		case 'u': {
 			unsigned int d =
 				(unsigned int) va_arg(va, unsigned int);
-			g_string_sprintfa(out, "%u", d);
+			g_string_append_printf(out, "%u", d);
 			break;
                 }
 		case 'l': {
 			long d = (long) va_arg(va, long);
 
 			if (*++str != 'd' && *str != 'u') {
-				g_string_sprintfa(out, "%ld", d);
+				g_string_append_printf(out, "%ld", d);
 				str--;
 			} else {
 				if (*str == 'd')
-					g_string_sprintfa(out, "%ld", d);
+					g_string_append_printf(out, "%ld", d);
 				else
-					g_string_sprintfa(out, "%lu", d);
+					g_string_append_printf(out, "%lu", d);
 			}
 			break;
 		}
 		default:
-			if (!format_expand_styles(out, *str)) {
+			adv = format_expand_styles(out, &str, &dest->flags);
+			if (!adv) {
 				g_string_append_c(out, '%');
 				g_string_append_c(out, *str);
+			} else {
+				str += adv - 1;
 			}
 			break;
 		}
@@ -250,10 +272,11 @@ static char *printtext_get_args(TEXT_DEST_REC *dest, const char *str,
 	return ret;
 }
 
-static char *printtext_expand_formats(const char *str)
+static char *printtext_expand_formats(const char *str, int *flags)
 {
 	GString *out;
 	char *ret;
+	int adv;
 
 	out = g_string_new(NULL);
 	for (; *str != '\0'; str++) {
@@ -265,9 +288,12 @@ static char *printtext_expand_formats(const char *str)
 		if (*++str == '\0')
 			break;
 
-		if (!format_expand_styles(out, *str)) {
+		adv = format_expand_styles(out, &str, flags);
+		if (!adv) {
 			g_string_append_c(out, '%');
 			g_string_append_c(out, *str);
+		} else {
+		     str += adv - 1;
 		}
 	}
 
@@ -276,7 +302,7 @@ static char *printtext_expand_formats(const char *str)
 	return ret;
 }
 
-void printtext_dest(TEXT_DEST_REC *dest, const char *text, va_list va)
+static void printtext_dest_args(TEXT_DEST_REC *dest, const char *text, va_list va)
 {
 	char *str;
 
@@ -291,6 +317,15 @@ void printtext_dest(TEXT_DEST_REC *dest, const char *text, va_list va)
 	g_free(str);
 }
 
+void printtext_dest(TEXT_DEST_REC *dest, const char *text, ...)
+{
+	va_list va;
+
+	va_start(va, text);
+	printtext_dest_args(dest, text, va);
+	va_end(va);
+}
+
 /* Write text to target - convert color codes */
 void printtext(void *server, const char *target, int level, const char *text, ...)
 {
@@ -302,7 +337,7 @@ void printtext(void *server, const char *target, int level, const char *text, ..
         format_create_dest(&dest, server, target, level, NULL);
 
 	va_start(va, text);
-	printtext_dest(&dest, text, va);
+	printtext_dest_args(&dest, text, va);
 	va_end(va);
 }
 
@@ -318,11 +353,33 @@ void printtext_string(void *server, const char *target, int level, const char *t
 
 	if (!sending_print_starting) {
 		sending_print_starting = TRUE;
-		signal_emit_id(signal_print_starting, 1, dest);
+		signal_emit_id(signal_print_starting, 1, &dest);
                 sending_print_starting = FALSE;
 	}
 
-        str = printtext_expand_formats(text);
+        str = printtext_expand_formats(text, &dest.flags);
+	print_line(&dest, str);
+        g_free(str);
+}
+
+/* Like printtext_window(), but don't handle %s etc. */
+void printtext_string_window(WINDOW_REC *window, int level, const char *text)
+{
+	TEXT_DEST_REC dest;
+        char *str;
+
+	g_return_if_fail(text != NULL);
+
+	format_create_dest(&dest, NULL, NULL, level,
+			   window != NULL ? window : active_win);
+
+	if (!sending_print_starting) {
+		sending_print_starting = TRUE;
+		signal_emit_id(signal_print_starting, 1, &dest);
+                sending_print_starting = FALSE;
+	}
+
+        str = printtext_expand_formats(text, &dest.flags);
 	print_line(&dest, str);
         g_free(str);
 }
@@ -338,7 +395,7 @@ void printtext_window(WINDOW_REC *window, int level, const char *text, ...)
 			   window != NULL ? window : active_win);
 
 	va_start(va, text);
-	printtext_dest(&dest, text, va);
+	printtext_dest_args(&dest, text, va);
 	va_end(va);
 }
 
@@ -351,14 +408,14 @@ void printtext_gui(const char *text)
 
         memset(&dest, 0, sizeof(dest));
 
-	str = printtext_expand_formats(text);
+	str = printtext_expand_formats(text, &dest.flags);
 	format_send_to_gui(&dest, str);
 	g_free(str);
 }
 
 static void msg_beep_check(TEXT_DEST_REC *dest)
 {
-	if (dest->level != 0 && (dest->level & MSGLEVEL_NOHILIGHT) == 0 &&
+	if (dest->level != 0 && (dest->level & MSGLEVEL_NO_ACT) == 0 &&
 	    (beep_msg_level & dest->level) &&
 	    (beep_when_away || (dest->server != NULL &&
 				!dest->server->usermode_away)) &&
@@ -369,31 +426,37 @@ static void msg_beep_check(TEXT_DEST_REC *dest)
 
 static void sig_print_text(TEXT_DEST_REC *dest, const char *text)
 {
+        THEME_REC *theme;
 	char *str, *tmp;
 
 	g_return_if_fail(dest != NULL);
 	g_return_if_fail(text != NULL);
 
+	if (dest->window == NULL) {
+                str = strip_codes(text);
+		printf("NO WINDOWS: %s\n", str);
+                g_free(str);
+                return;
+	}
+
 	msg_beep_check(dest);
 
-	dest->window->last_line = time(NULL);
-	format_newline(dest->window);
+        if ((dest->level & MSGLEVEL_NEVER) == 0)
+		dest->window->last_line = time(NULL);
 
 	/* add timestamp/server tag here - if it's done in print_line()
 	   it would be written to log files too */
-	tmp = format_get_line_start(current_theme, dest, time(NULL));
-	str = format_add_linestart(text, tmp);
+        theme = window_get_theme(dest->window);
+	tmp = format_get_line_start(theme, dest, time(NULL));
+	str = !theme->info_eol ? format_add_linestart(text, tmp) :
+		format_add_lineend(text, tmp);
+
 	g_free_not_null(tmp);
 
 	format_send_to_gui(dest, str);
 	g_free(str);
 
-	signal_emit_id(signal_print_text_finished, 1, dest->window);
-}
-
-static void sig_print_text_free(TEXT_DEST_REC *dest, const char *text)
-{
-        g_free_and_null(dest->hilight_color);
+	signal_emit_id(signal_gui_print_text_finished, 1, dest->window);
 }
 
 void printtext_multiline(void *server, const char *target, int level,
@@ -414,10 +477,10 @@ static void sig_gui_dialog(const char *type, const char *text)
 {
 	char *format;
 
-	if (g_strcasecmp(type, "warning") == 0)
-		format = _("%_Warning:%_ %s");
-	else if (g_strcasecmp(type, "error") == 0)
-		format = _("%_Error:%_ %s");
+	if (g_ascii_strcasecmp(type, "warning") == 0)
+		format = "%_Warning:%_ %s";
+	else if (g_ascii_strcasecmp(type, "error") == 0)
+		format = "%_Error:%_ %s";
 	else
 		format = "%s";
 
@@ -426,7 +489,7 @@ static void sig_gui_dialog(const char *type, const char *text)
 
 static void read_settings(void)
 {
-	beep_msg_level = level2bits(settings_get_str("beep_msg_level"));
+	beep_msg_level = settings_get_level("beep_msg_level");
 	beep_when_away = settings_get_bool("beep_when_away");
         beep_when_window_active = settings_get_bool("beep_when_window_active");
 }
@@ -434,16 +497,13 @@ static void read_settings(void)
 void printtext_init(void)
 {
 	sending_print_starting = FALSE;
-	signal_gui_print_text = signal_get_uniq_id("gui print text");
-	signal_print_text_stripped = signal_get_uniq_id("print text stripped");
-	signal_print_text = signal_get_uniq_id("print text");
-	signal_print_text_finished = signal_get_uniq_id("print text finished");
-	signal_print_format = signal_get_uniq_id("print format");
+	signal_gui_print_text_finished = signal_get_uniq_id("gui print text finished");
 	signal_print_starting = signal_get_uniq_id("print starting");
+	signal_print_text = signal_get_uniq_id("print text");
+	signal_print_format = signal_get_uniq_id("print format");
 
 	read_settings();
 	signal_add("print text", (SIGNAL_FUNC) sig_print_text);
-	signal_add_last("print text", (SIGNAL_FUNC) sig_print_text_free);
 	signal_add("gui dialog", (SIGNAL_FUNC) sig_gui_dialog);
 	signal_add("setup changed", (SIGNAL_FUNC) read_settings);
 }
@@ -451,7 +511,6 @@ void printtext_init(void)
 void printtext_deinit(void)
 {
 	signal_remove("print text", (SIGNAL_FUNC) sig_print_text);
-	signal_remove("print text", (SIGNAL_FUNC) sig_print_text_free);
 	signal_remove("gui dialog", (SIGNAL_FUNC) sig_gui_dialog);
 	signal_remove("setup changed", (SIGNAL_FUNC) read_settings);
 }
