@@ -13,9 +13,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
 #include "module.h"
@@ -115,8 +115,8 @@ void theme_destroy(THEME_REC *rec)
 }
 
 static char *theme_replace_expand(THEME_REC *theme, int index,
-				  char default_fg, char default_bg,
-				  char *last_fg, char *last_bg,
+				  theme_rm_col default_fg, theme_rm_col default_bg,
+				  theme_rm_col *last_fg, theme_rm_col *last_bg,
 				  char chr, int flags)
 {
 	GSList *rec;
@@ -168,15 +168,44 @@ static void theme_format_append_variable(GString *str, const char **format)
 	g_free(value);
 }
 
+static inline int chr_is_valid_rgb(const char format[])
+{
+	int tmp;
+	for (tmp = 1; tmp < 7; ++tmp) {
+		if (!isxdigit(format[tmp]))
+			return tmp;
+	}
+	return 0;
+}
+
+static inline int chr_is_valid_ext(const char format[])
+{
+	if (format[1] < '0' || format[1] > '7')
+		return 1;
+
+	if (format[1] == '7') {
+		if (!isalpha(format[2]) || format[2] == 'y' || format[2] == 'Y'
+		    || format[2] =='z' || format[2] == 'Z')
+			return 2;
+	} else if (format[1] == '0') {
+		if (!isxdigit(format[2]))
+			return 2;
+	} else if (!isalnum(format[2]))
+		return 2;
+
+	return 0;
+}
+
 /* append next "item", either a character, $variable or %format */
 static void theme_format_append_next(THEME_REC *theme, GString *str,
 				     const char **format,
-				     char default_fg, char default_bg,
-				     char *last_fg, char *last_bg,
+				     theme_rm_col default_fg, theme_rm_col default_bg,
+				     theme_rm_col *last_fg, theme_rm_col *last_bg,
 				     int flags)
 {
 	int index;
 	unsigned char chr;
+	char *t;
 
 	chr = **format;
 	if ((chr == '$' || chr == '%') &&
@@ -203,22 +232,50 @@ static void theme_format_append_next(THEME_REC *theme, GString *str,
 				/* %n = change to default color */
 				g_string_append(str, "%n");
 
-				if (default_bg != 'n') {
+				if (default_bg.m[0] != 'n') {
 					g_string_append_c(str, '%');
-					g_string_append_c(str, default_bg);
+					g_string_append(str, default_bg.m);
 				}
-				if (default_fg != 'n') {
+				if (default_fg.m[0] != 'n') {
 					g_string_append_c(str, '%');
-					g_string_append_c(str, default_fg);
+					g_string_append(str, default_fg.m);
 				}
 
 				*last_fg = default_fg;
 				*last_bg = default_bg;
+			} else if (chr == 'z' || chr == 'Z') {
+				if (chr_is_valid_rgb(*format) == 0) {
+					t = chr == 'z' ? (*last_bg).m : (*last_fg).m;
+					strncpy(t, *format, 7);
+					t[7] = '\0';
+					g_string_append_c(str, '%');
+					g_string_append(str, t);
+					(*format)+=6;
+				} else {
+					g_string_append(str, "%%");
+					g_string_append_c(str, **format);
+				}
+			} else if (chr == 'x' || chr == 'X') {
+				if (chr_is_valid_ext(*format) == 0) {
+					t = chr == 'x' ? (*last_bg).m : (*last_fg).m;
+					strncpy(t, *format, 3);
+					t[3] = '\0';
+					g_string_append_c(str, '%');
+					g_string_append(str, t);
+					(*format)+=2;
+				} else {
+					g_string_append(str, "%%");
+					g_string_append_c(str, **format);
+				}
 			} else {
-				if (IS_FGCOLOR_FORMAT(chr))
-					*last_fg = chr;
-				if (IS_BGCOLOR_FORMAT(chr))
-					*last_bg = chr;
+				if (IS_FGCOLOR_FORMAT(chr)) {
+					(*last_fg).m[0] = chr;
+					(*last_fg).m[1] = '\0';
+				}
+				if (IS_BGCOLOR_FORMAT(chr)) {
+					(*last_bg).m[0] = chr;
+					(*last_bg).m[1] = '\0';
+				}
 				g_string_append_c(str, '%');
 				g_string_append_c(str, chr);
 			}
@@ -306,7 +363,10 @@ static int data_is_empty(const char **data)
 char *theme_format_expand_get(THEME_REC *theme, const char **format)
 {
 	GString *str;
-	char *ret, dummy;
+	char *ret;
+	theme_rm_col dummy, reset;
+	dummy.m[0] = '\0';
+	strcpy(reset.m, "n");
 	int braces = 1; /* we start with one brace opened */
 
 	str = g_string_new(NULL);
@@ -321,11 +381,11 @@ char *theme_format_expand_get(THEME_REC *theme, const char **format)
 			continue;
 		} else {
 			theme_format_append_next(theme, str, format,
-						 'n', 'n',
+						 reset, reset,
 						 &dummy, &dummy, 0);
 			continue;
 		}
-		
+
 		if (braces == 0) {
 			(*format)++;
 			break;
@@ -343,15 +403,19 @@ char *theme_format_expand_get(THEME_REC *theme, const char **format)
 /* expand a single {abstract ...data... } */
 static char *theme_format_expand_abstract(THEME_REC *theme,
 					  const char **formatp,
-					  char default_fg, char default_bg,
+					  theme_rm_col *last_fg,
+					  theme_rm_col *last_bg,
 					  int flags)
 {
 	GString *str;
 	const char *p, *format;
 	char *abstract, *data, *ret;
+	theme_rm_col default_fg, default_bg;
 	int len;
 
 	format = *formatp;
+	default_fg = *last_fg;
+	default_bg = *last_bg;
 
 	/* get abstract name first */
 	p = format;
@@ -425,7 +489,7 @@ static char *theme_format_expand_abstract(THEME_REC *theme,
 	/* abstract may itself contain abstracts or replaces */
 	p = abstract;
 	ret = theme_format_expand_data(theme, &p, default_fg, default_bg,
-				       &default_fg, &default_bg,
+				       last_fg, last_bg,
 				       flags | EXPAND_FLAG_LASTCOLOR_ARG);
 	g_free(abstract);
 	return ret;
@@ -433,13 +497,13 @@ static char *theme_format_expand_abstract(THEME_REC *theme,
 
 /* expand the data part in {abstract data} */
 char *theme_format_expand_data(THEME_REC *theme, const char **format,
-			       char default_fg, char default_bg,
-			       char *save_last_fg, char *save_last_bg,
+			       theme_rm_col default_fg, theme_rm_col default_bg,
+			       theme_rm_col *save_last_fg, theme_rm_col *save_last_bg,
 			       int flags)
 {
 	GString *str;
 	char *ret, *abstract;
-	char last_fg, last_bg;
+	theme_rm_col last_fg, last_bg;
         int recurse_flags;
 
 	last_fg = default_fg;
@@ -482,7 +546,7 @@ char *theme_format_expand_data(THEME_REC *theme, const char **format,
 
 		/* get a single {...} */
 		abstract = theme_format_expand_abstract(theme, format,
-							last_fg, last_bg,
+							&last_fg, &last_bg,
 							recurse_flags);
 		if (abstract != NULL) {
 			g_string_append(str, abstract);
@@ -490,13 +554,11 @@ char *theme_format_expand_data(THEME_REC *theme, const char **format,
 		}
 	}
 
-	if ((flags & EXPAND_FLAG_LASTCOLOR_ARG) == 0) {
 		/* save the last color */
 		if (save_last_fg != NULL)
 			*save_last_fg = last_fg;
 		if (save_last_bg != NULL)
 			*save_last_bg = last_bg;
-	}
 
 	ret = str->str;
         g_string_free(str, FALSE);
@@ -510,7 +572,8 @@ char *theme_format_expand_data(THEME_REC *theme, const char **format,
 static char *theme_format_compress_colors(THEME_REC *theme, const char *format)
 {
 	GString *str;
-	char *ret, last_fg, last_bg;
+	char *ret;
+	char last_fg, last_bg;
 
 	str = g_string_new(NULL);
 
@@ -543,8 +606,12 @@ static char *theme_format_compress_colors(THEME_REC *theme, const char *format)
 
 				if (IS_FGCOLOR_FORMAT(*format))
 					last_fg = *format;
+				else if (*format == 'Z' || *format == 'X')
+					last_fg = '\0';
 				if (IS_BGCOLOR_FORMAT(*format))
 					last_bg = *format;
+				else if (*format == 'z' || *format == 'x')
+					last_bg = '\0';
 			}
 			format++;
 		}
@@ -558,11 +625,13 @@ static char *theme_format_compress_colors(THEME_REC *theme, const char *format)
 char *theme_format_expand(THEME_REC *theme, const char *format)
 {
 	char *data, *ret;
+	theme_rm_col reset;
+	strcpy(reset.m, "n");
 
 	g_return_val_if_fail(theme != NULL, NULL);
 	g_return_val_if_fail(format != NULL, NULL);
 
-	data = theme_format_expand_data(theme, &format, 'n', 'n', NULL, NULL,
+	data = theme_format_expand_data(theme, &format, reset, reset, NULL, NULL,
 					EXPAND_FLAG_ROOT);
 	ret = theme_format_compress_colors(theme, data);
         g_free(data);
@@ -799,7 +868,7 @@ static THEME_REC *theme_find(const char *name)
 	for (tmp = themes; tmp != NULL; tmp = tmp->next) {
 		THEME_REC *rec = tmp->data;
 
-		if (g_strcasecmp(rec->name, name) == 0)
+		if (g_ascii_strcasecmp(rec->name, name) == 0)
 			return rec;
 	}
 
@@ -944,10 +1013,6 @@ static int theme_read(THEME_REC *theme, const char *path)
 		config_get_int(config, NULL, "default_color", -1);
 	theme->info_eol = config_get_bool(config, NULL, "info_eol", FALSE);
 
-	/* FIXME: remove after 0.7.99 */
-	if (theme->default_color == 0 &&
-	    config_get_int(config, NULL, "default_real_color", -1) != -1)
-                theme->default_color = -1;
 	theme_read_replaces(config, theme);
 
 	if (path != NULL)
@@ -970,7 +1035,7 @@ typedef struct {
 
 static int theme_search_equal(THEME_SEARCH_REC *r1, THEME_SEARCH_REC *r2)
 {
-	return g_strcasecmp(r1->short_name, r2->short_name);
+	return g_ascii_strcasecmp(r1->short_name, r2->short_name);
 }
 
 static void theme_get_modules(char *module, FORMAT_REC *formats, GSList **list)
@@ -1001,7 +1066,7 @@ static THEME_SEARCH_REC *theme_search(GSList *list, const char *module)
 	while (list != NULL) {
 		rec = list->data;
 
-		if (g_strcasecmp(rec->short_name, module) == 0)
+		if (g_ascii_strcasecmp(rec->short_name, module) == 0)
 			return rec;
 		list = list->next;
 	}
@@ -1026,7 +1091,7 @@ static void theme_show(THEME_SEARCH_REC *rec, const char *key, const char *value
 
 		if (formats[n].tag == NULL)
 			last_title = text;
-		else if ((value != NULL && key != NULL && g_strcasecmp(formats[n].tag, key) == 0) ||
+		else if ((value != NULL && key != NULL && g_ascii_strcasecmp(formats[n].tag, key) == 0) ||
 			 (value == NULL && (key == NULL || stristr(formats[n].tag, key) != NULL))) {
 			if (first) {
 				printformat(NULL, NULL, MSGLEVEL_CLIENTCRAP, TXT_FORMAT_TITLE, rec->short_name, formats[0].def);
@@ -1086,7 +1151,7 @@ static void cmd_format(const char *data)
 	for (tmp = modules; tmp != NULL; tmp = tmp->next) {
 		THEME_SEARCH_REC *rec = tmp->data;
 
-		if (module == NULL || g_strcasecmp(rec->short_name, module) == 0)
+		if (module == NULL || g_ascii_strcasecmp(rec->short_name, module) == 0)
 			theme_show(rec, key, value, reset);
 	}
 	g_slist_foreach(modules, (GFunc) g_free, NULL);
@@ -1138,13 +1203,14 @@ static void theme_save(THEME_REC *theme, int save_all)
 	CONFIG_REC *config;
 	THEME_SAVE_REC data;
 	char *path;
+	char *basename;
 	int ok;
 
 	config = config_open(theme->path, -1);
         if (config != NULL)
                 config_parse(config);
         else {
-                if (g_strcasecmp(theme->name, "default") == 0) {
+                if (g_ascii_strcasecmp(theme->name, "default") == 0) {
                         config = config_open(NULL, -1);
                         config_parse_data(config, default_theme, "internal");
                         config_change_file_name(config, theme->path, 0660);
@@ -1160,10 +1226,11 @@ static void theme_save(THEME_REC *theme, int save_all)
         data.save_all = save_all;
 	g_hash_table_foreach(theme->modules, (GHFunc) module_save, &data);
 
+	basename = g_path_get_basename(theme->path);
         /* always save the theme to ~/.irssi/ */
-	path = g_strdup_printf("%s/%s", get_irssi_dir(),
-			       g_basename(theme->path));
+	path = g_strdup_printf("%s/%s", get_irssi_dir(), basename);
 	ok = config_write(config, path, 0660) == 0;
+	g_free(basename);
 
 	printformat(NULL, NULL, MSGLEVEL_CLIENTNOTICE,
 		    ok ? TXT_THEME_SAVED : TXT_THEME_SAVE_FAILED,
@@ -1207,7 +1274,7 @@ static void complete_format_list(THEME_SEARCH_REC *rec, const char *key, GList *
 	for (n = 1; formats[n].def != NULL; n++) {
 		const char *item = formats[n].tag;
 
-		if (item != NULL && g_strncasecmp(item, key, len) == 0)
+		if (item != NULL && g_ascii_strncasecmp(item, key, len) == 0)
                         *list = g_list_append(*list, g_strdup(item));
 	}
 }
@@ -1226,7 +1293,7 @@ static GList *completion_get_formats(const char *module, const char *key)
 		for (tmp = modules; tmp != NULL; tmp = tmp->next) {
 			THEME_SEARCH_REC *rec = tmp->data;
 
-			if (*module == '\0' || g_strcasecmp(rec->short_name, module) == 0)
+			if (*module == '\0' || g_ascii_strcasecmp(rec->short_name, module) == 0)
 				complete_format_list(rec, key, &list);
 		}
 	}
@@ -1339,6 +1406,8 @@ static THEME_REC *read_internal_theme(void)
 	THEME_REC *theme;
 
 	theme = theme_create("internal", "_internal");
+	theme->refcount++;
+	theme_destroy(theme);
 
 	config = config_open(NULL, -1);
 	config_parse_data(config, default_theme, "internal");
@@ -1359,7 +1428,6 @@ void themes_init(void)
         init_finished = FALSE;
         init_errors = NULL;
 
-	themes = NULL;
 	themes_reload();
 
 	command_bind("format", NULL, (SIGNAL_FUNC) cmd_format);
