@@ -68,9 +68,11 @@ static void sig_layout_restore_item(WINDOW_REC *window, const char *type,
 			   (SIGNAL_FUNC) signal_query_created_curwin);
 
                 restore_win = window;
-		
+
 		protocol = chat_protocol_find(chat_type);
-		if (protocol->query_create != NULL)
+		if (protocol == NULL)
+			window_bind_add(window, tag, name);
+		else if (protocol->query_create != NULL)
 			protocol->query_create(tag, name, TRUE);
 		else {
 			QUERY_REC *query;
