@@ -13,9 +13,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
 #include "module.h"
@@ -25,7 +25,7 @@
 #include "ignore.h"
 #include "settings.h"
 
-#include "irc.h"
+#include "irc-servers.h"
 #include "irc-channels.h"
 #include "irc-queries.h"
 
@@ -37,24 +37,57 @@
 #include "fe-queries.h"
 #include "window-items.h"
 
+static const char *skip_target(IRC_SERVER_REC *server, const char *target)
+{
+	int i = 0;
+	const char *val, *chars;
+
+	/* Quick check */
+	if (server == NULL || server->prefix[(int)(unsigned char)*target] == 0)
+		return target;
+
+	/* Hack: for bahamut 1.4 which sends neither STATUSMSG nor
+	 * WALLCHOPS in 005, accept @#chan and @+#chan (but not +#chan) */
+	val = g_hash_table_lookup(server->isupport, "STATUSMSG");
+	if (val == NULL && *target != '@')
+		return target;
+	chars = val ? val : "@+";
+	for(i = 0; target[i] != '\0'; i++) {
+		if (strchr(chars, target[i]) == NULL)
+			break;
+	};
+
+	if(ischannel(target[i]))
+		target += i;
+
+	return target;
+}
+
 static void sig_message_own_public(SERVER_REC *server, const char *msg,
 				   const char *target, const char *origtarget)
 {
+	const char *oldtarget;
 	char *nickmode;
 
-	if (IS_IRC_SERVER(server) && target != NULL &&
-	    *target == '@' && ischannel(target[1])) {
-		/* Hybrid 6 feature, send msg to all ops in channel */
-		nickmode = channel_get_nickmode(channel_find(server, target+1),
+	if (!IS_IRC_SERVER(server))
+		return;
+	oldtarget = target;
+	target = skip_target(IRC_SERVER(server), target);
+	if (target != oldtarget) {
+		/* Hybrid 6 / Bahamut feature, send msg to all
+		   ops / ops+voices in channel */
+		nickmode = channel_get_nickmode(channel_find(server, target),
 						server->nick);
 
-		printformat_module("fe-common/core", server, target+1,
+		printformat_module("fe-common/core", server, target,
 				   MSGLEVEL_PUBLIC | MSGLEVEL_NOHILIGHT |
 				   MSGLEVEL_NO_ACT,
 				   TXT_OWN_MSG_CHANNEL,
-				   server->nick, target, msg, nickmode);
+				   server->nick, oldtarget, msg, nickmode);
+		g_free(nickmode);
                 signal_stop();
 	}
+
 }
 
 /* received msg to all ops in channel */
@@ -65,13 +98,14 @@ static void sig_message_irc_op_public(SERVER_REC *server, const char *msg,
 	char *nickmode, *optarget;
 
 	nickmode = channel_get_nickmode(channel_find(server, target),
-					server->nick);
+					nick);
 
         optarget = g_strconcat("@", target, NULL);
 	printformat_module("fe-common/core", server, target,
-			   MSGLEVEL_PUBLIC | MSGLEVEL_HILIGHT,
-			   TXT_PUBMSG_ME_CHANNEL,
+			   MSGLEVEL_PUBLIC,
+			   TXT_PUBMSG_CHANNEL,
 			   nick, optarget, msg, nickmode);
+	g_free(nickmode);
         g_free(optarget);
 }
 
@@ -89,6 +123,7 @@ static void sig_message_own_wall(SERVER_REC *server, const char *msg,
 			   MSGLEVEL_NO_ACT,
 			   TXT_OWN_MSG_CHANNEL,
 			   server->nick, optarget, msg, nickmode);
+	g_free(nickmode);
         g_free(optarget);
 }
 
@@ -96,8 +131,11 @@ static void sig_message_own_action(IRC_SERVER_REC *server, const char *msg,
                                    const char *target)
 {
 	void *item;
+	const char *oldtarget;
         char *freemsg = NULL;
 
+	oldtarget = target;
+	target = skip_target(IRC_SERVER(server), target);
         if (ischannel(*target))
 		item = irc_channel_find(server, target);
 	else
@@ -109,9 +147,8 @@ static void sig_message_own_action(IRC_SERVER_REC *server, const char *msg,
 	printformat(server, target,
 		    MSGLEVEL_ACTIONS | MSGLEVEL_NOHILIGHT | MSGLEVEL_NO_ACT |
 		    (ischannel(*target) ? MSGLEVEL_PUBLIC : MSGLEVEL_MSGS),
-		    item != NULL ? IRCTXT_OWN_ACTION : IRCTXT_OWN_ACTION_TARGET,
-		    server->nick, msg, target);
-
+		    item != NULL && oldtarget == target ? IRCTXT_OWN_ACTION : IRCTXT_OWN_ACTION_TARGET,
+		    server->nick, msg, oldtarget);
         g_free_not_null(freemsg);
 }
 
@@ -120,15 +157,23 @@ static void sig_message_irc_action(IRC_SERVER_REC *server, const char *msg,
 				   const char *target)
 {
 	void *item;
+	const char *oldtarget;
         char *freemsg = NULL;
 	int level;
 
+	oldtarget = target;
+	target = skip_target(IRC_SERVER(server), target);
+
 	level = MSGLEVEL_ACTIONS |
 		(ischannel(*target) ? MSGLEVEL_PUBLIC : MSGLEVEL_MSGS);
 
 	if (ignore_check(SERVER(server), nick, address, target, msg, level))
 		return;
 
+	if (ignore_check(SERVER(server), nick, address, target, msg,
+			 level | MSGLEVEL_NO_ACT))
+		level |= MSGLEVEL_NO_ACT;
+
 	if (ischannel(*target))
 		item = irc_channel_find(server, target);
         else
@@ -139,15 +184,15 @@ static void sig_message_irc_action(IRC_SERVER_REC *server, const char *msg,
 
 	if (ischannel(*target)) {
 		/* channel action */
-		if (window_item_is_active(item)) {
+		if (window_item_is_active(item) && target == oldtarget) {
 			/* message to active channel in window */
 			printformat(server, target, level,
 				    IRCTXT_ACTION_PUBLIC, nick, msg);
 		} else {
-			/* message to not existing/active channel */
+			/* message to not existing/active channel, or to @/+ */
 			printformat(server, target, level,
 				    IRCTXT_ACTION_PUBLIC_CHANNEL,
-				    nick, target, msg);
+				    nick, oldtarget, msg);
 		}
 	} else {
 		/* private action */
@@ -157,13 +202,13 @@ static void sig_message_irc_action(IRC_SERVER_REC *server, const char *msg,
 			    nick, address == NULL ? "" : address, msg);
 	}
 
-        g_free_not_null(freemsg);
+	g_free_not_null(freemsg);
 }
 
 static void sig_message_own_notice(IRC_SERVER_REC *server, const char *msg,
 				   const char *target)
 {
-	printformat(server, target, MSGLEVEL_NOTICES |
+	printformat(server, skip_target(server, target), MSGLEVEL_NOTICES |
 		    MSGLEVEL_NOHILIGHT | MSGLEVEL_NO_ACT,
 		    IRCTXT_OWN_NOTICE, target, msg);
 }
@@ -172,9 +217,13 @@ static void sig_message_irc_notice(SERVER_REC *server, const char *msg,
 				   const char *nick, const char *address,
 				   const char *target)
 {
-	int op_notice;
+	const char *oldtarget;
+	int level = MSGLEVEL_NOTICES;
 
-	if (address == NULL) {
+	oldtarget = target;
+	target = skip_target(IRC_SERVER(server), target);
+
+	if (address == NULL || *address == '\0') {
 		/* notice from server */
 		if (!ignore_check(server, nick, "",
 				  target, msg, MSGLEVEL_SNOTES)) {
@@ -184,24 +233,25 @@ static void sig_message_irc_notice(SERVER_REC *server, const char *msg,
                 return;
 	}
 
-	op_notice = *target == '@' && ischannel(target[1]);
-	if (op_notice) target++;
+	if (ignore_check(server, nick, address,
+			 ischannel(*target) ? target : NULL,
+			 msg, level))
+		return;
 
 	if (ignore_check(server, nick, address,
 			 ischannel(*target) ? target : NULL,
-			 msg, MSGLEVEL_NOTICES))
-		return;
+			 msg, level | MSGLEVEL_NO_ACT))
+		level |= MSGLEVEL_NO_ACT;
 
-	if (ischannel(*target)) {
+        if (ischannel(*target)) {
 		/* notice in some channel */
-		printformat(server, target, MSGLEVEL_NOTICES,
-			    op_notice ? IRCTXT_NOTICE_PUBLIC_OPS :
-			    IRCTXT_NOTICE_PUBLIC, nick, target, msg);
+		printformat(server, target, level,
+			    IRCTXT_NOTICE_PUBLIC, nick, oldtarget, msg);
 	} else {
 		/* private notice */
 		privmsg_get_query(SERVER(server), nick, FALSE,
 				  MSGLEVEL_NOTICES);
-		printformat(server, nick, MSGLEVEL_NOTICES,
+		printformat(server, nick, level,
 			    IRCTXT_NOTICE_PRIVATE, nick, address, msg);
 	}
 }
@@ -209,30 +259,34 @@ static void sig_message_irc_notice(SERVER_REC *server, const char *msg,
 static void sig_message_own_ctcp(IRC_SERVER_REC *server, const char *cmd,
 				 const char *data, const char *target)
 {
-	printformat(server, target, MSGLEVEL_CTCPS |
+	printformat(server, skip_target(server, target), MSGLEVEL_CTCPS |
 		    MSGLEVEL_NOHILIGHT | MSGLEVEL_NO_ACT,
 		    IRCTXT_OWN_CTCP, target, cmd, data);
 }
 
-static void sig_message_irc_ctcp(IRC_SERVER_REC *server, const char *msg,
-				 const char *nick, const char *addr,
-				 const char *target)
+static void sig_message_irc_ctcp(IRC_SERVER_REC *server, const char *cmd,
+				 const char *data, const char *nick,
+				 const char *addr, const char *target)
 {
+	const char *oldtarget;
+
+	oldtarget = target;
+	target = skip_target(server, target);
 	printformat(server, ischannel(*target) ? target : nick, MSGLEVEL_CTCPS,
-		    IRCTXT_CTCP_REQUESTED, nick, addr, msg, target);
+		    IRCTXT_CTCP_REQUESTED, nick, addr, cmd, data, oldtarget);
 }
 
 void fe_irc_messages_init(void)
 {
-        signal_add("message own_public", (SIGNAL_FUNC) sig_message_own_public);
-        signal_add("message irc op_public", (SIGNAL_FUNC) sig_message_irc_op_public);
-        signal_add("message irc own_wall", (SIGNAL_FUNC) sig_message_own_wall);
-        signal_add("message irc own_action", (SIGNAL_FUNC) sig_message_own_action);
-        signal_add("message irc action", (SIGNAL_FUNC) sig_message_irc_action);
-        signal_add("message irc own_notice", (SIGNAL_FUNC) sig_message_own_notice);
-        signal_add("message irc notice", (SIGNAL_FUNC) sig_message_irc_notice);
-        signal_add("message irc own_ctcp", (SIGNAL_FUNC) sig_message_own_ctcp);
-        signal_add("message irc ctcp", (SIGNAL_FUNC) sig_message_irc_ctcp);
+        signal_add_last("message own_public", (SIGNAL_FUNC) sig_message_own_public);
+        signal_add_last("message irc op_public", (SIGNAL_FUNC) sig_message_irc_op_public);
+        signal_add_last("message irc own_wall", (SIGNAL_FUNC) sig_message_own_wall);
+        signal_add_last("message irc own_action", (SIGNAL_FUNC) sig_message_own_action);
+        signal_add_last("message irc action", (SIGNAL_FUNC) sig_message_irc_action);
+        signal_add_last("message irc own_notice", (SIGNAL_FUNC) sig_message_own_notice);
+        signal_add_last("message irc notice", (SIGNAL_FUNC) sig_message_irc_notice);
+        signal_add_last("message irc own_ctcp", (SIGNAL_FUNC) sig_message_own_ctcp);
+        signal_add_last("message irc ctcp", (SIGNAL_FUNC) sig_message_irc_ctcp);
 }
 
 void fe_irc_messages_deinit(void)
