@@ -22,9 +22,12 @@
 #include "signals.h"
 #include "term.h"
 #include "terminfo-core.h"
+#include "fe-windows.h"
 #include "utf8.h"
 
 #include <signal.h>
+#include <termios.h>
+#include <stdio.h>
 
 /* returns number of characters in the beginning of the buffer being a
    a single character, or -1 if more input is needed. The character will be
@@ -48,7 +51,8 @@ static int vcmove, vcx, vcy, curs_visible;
 static int crealx, crealy, cforcemove;
 static int curs_x, curs_y;
 
-static int last_fg, last_bg, last_attrs;
+static unsigned int last_fg, last_bg;
+static int last_attrs;
 
 static GSource *sigcont_source;
 static volatile sig_atomic_t got_sigcont;
@@ -136,7 +140,7 @@ int term_init(void)
 
         term_set_input_type(TERM_TYPE_8BIT);
 	term_common_init();
-        g_atexit(term_deinit);
+        atexit(term_deinit);
         return TRUE;
 }
 
@@ -289,53 +293,104 @@ void term_window_scroll(TERM_WINDOW *window, int count)
         term_move_reset(vcx, vcy);
 
         /* set the newly scrolled area dirty */
-	for (y = 0; y < window->height; y++)
+	for (y = 0; (window->y+y) < term_height && y < window->height; y++)
 		term_lines_empty[window->y+y] = FALSE;
 }
 
+inline static int term_putchar(int c)
+{
+        return fputc(c, current_term->out);
+}
+
+/* copied from terminfo-core.c */
+int tputs();
+
+static int termctl_set_color_24bit(int bg, unsigned int lc)
+{
+	static char buf[20];
+	const unsigned char color[] = { lc >> 16, lc >> 8, lc };
+
+	if (!term_use_colors24) {
+		if (bg)
+			terminfo_set_bg(color_24bit_256(color));
+		else
+			terminfo_set_fg(color_24bit_256(color));
+		return -1;
+	}
+
+	/* \e[x8;2;...;...;...m */
+	sprintf(buf, "\033[%d8;2;%d;%d;%dm", bg ? 4 : 3, color[0], color[1], color[2]);
+	return tputs(buf, 0, term_putchar);
+}
+
+#define COLOR_RESET UINT_MAX
+#define COLOR_BLACK24 COLOR_RESET - 1
+
 /* Change active color */
+#ifdef TERM_TRUECOLOR
+void term_set_color2(TERM_WINDOW *window, int col, unsigned int fgcol24, unsigned int bgcol24)
+#else
 void term_set_color(TERM_WINDOW *window, int col)
+#endif
 {
 	int set_normal;
-	int fg = col & 0x0f;
-	int bg = (col & 0xf0) >> 4;
 
-        set_normal = ((col & ATTR_RESETFG) && last_fg != -1) ||
-		((col & ATTR_RESETBG) && last_bg != -1);
+	unsigned int fg, bg;
+#ifdef TERM_TRUECOLOR
+	if (col & ATTR_FGCOLOR24) {
+		if (fgcol24)
+			fg = fgcol24 << 8;
+		else
+			fg = COLOR_BLACK24;
+	} else
+#endif
+		fg = (col & FG_MASK);
+
+#ifdef TERM_TRUECOLOR
+	if (col & ATTR_BGCOLOR24) {
+		if (bgcol24)
+			bg = bgcol24 << 8;
+		else
+			bg = COLOR_BLACK24;
+	} else
+#endif
+		bg = ((col & BG_MASK) >> BG_SHIFT);
+
+	if (!term_use_colors && bg > 0)
+		col |= ATTR_REVERSE;
+
+        set_normal = ((col & ATTR_RESETFG) && last_fg != COLOR_RESET) ||
+		((col & ATTR_RESETBG) && last_bg != COLOR_RESET);
 	if (((last_attrs & ATTR_BOLD) && (col & ATTR_BOLD) == 0) ||
+	    ((last_attrs & ATTR_REVERSE) && (col & ATTR_REVERSE) == 0) ||
 	    ((last_attrs & ATTR_BLINK) && (col & ATTR_BLINK) == 0)) {
-		/* we'll need to get rid of bold/blink - this can only be
-		   done with setting the default color */
+		/* we'll need to get rid of bold/blink/reverse - this
+		   can only be done with setting the default color */
 		set_normal = TRUE;
 	}
 
 	if (set_normal) {
-		last_fg = last_bg = -1;
+		last_fg = last_bg = COLOR_RESET;
                 last_attrs = 0;
 		terminfo_set_normal();
 	}
 
-	if (!term_use_colors && (col & 0xf0) != 0)
-		col |= ATTR_REVERSE;
-
-	/* reversed text (use standout) */
-	if (col & ATTR_REVERSE) {
-		if ((last_attrs & ATTR_REVERSE) == 0)
-			terminfo_set_standout(TRUE);
-	} else if (last_attrs & ATTR_REVERSE)
-		terminfo_set_standout(FALSE);
-
 	/* set foreground color */
 	if (fg != last_fg &&
 	    (fg != 0 || (col & ATTR_RESETFG) == 0)) {
                 if (term_use_colors) {
 			last_fg = fg;
-			terminfo_set_fg(last_fg);
+			if (fg >> 8)
+				termctl_set_color_24bit(0,
+							last_fg == COLOR_BLACK24 ? 0
+							: last_fg >> 8);
+			else
+				terminfo_set_fg(last_fg);
 		}
 	}
 
 	/* set background color */
-	if (col & 0x80 && window->term->TI_colors == 8)
+	if (window && (term_color256map[bg&0xff]&8) == window->term->TI_colors)
 		col |= ATTR_BLINK;
 	if (col & ATTR_BLINK)
 		current_term->set_blink(current_term);
@@ -344,12 +399,21 @@ void term_set_color(TERM_WINDOW *window, int col)
 	    (bg != 0 || (col & ATTR_RESETBG) == 0)) {
                 if (term_use_colors) {
 			last_bg = bg;
-			terminfo_set_bg(last_bg);
+			if (bg >> 8)
+				termctl_set_color_24bit(1,
+							last_bg == COLOR_BLACK24 ? 0
+							: last_bg >> 8);
+			else
+				terminfo_set_bg(last_bg);
 		}
 	}
 
+	/* reversed text */
+	if (col & ATTR_REVERSE)
+		terminfo_set_reverse();
+
 	/* bold */
-	if (col & 0x08 && window->term->TI_colors == 8)
+	if (window && (term_color256map[fg&0xff]&8) == window->term->TI_colors)
 		col |= ATTR_BOLD;
 	if (col & ATTR_BOLD)
 		terminfo_set_bold();
@@ -361,7 +425,15 @@ void term_set_color(TERM_WINDOW *window, int col)
 	} else if (last_attrs & ATTR_UNDERLINE)
 		terminfo_set_uline(FALSE);
 
-        last_attrs = col & ~0xff;
+	/* italic */
+	if (col & ATTR_ITALIC) {
+		if ((last_attrs & ATTR_ITALIC) == 0)
+			terminfo_set_italic(TRUE);
+	} else if (last_attrs & ATTR_ITALIC)
+		terminfo_set_italic(FALSE);
+
+	/* update the new attribute settings whilst ignoring color values.  */
+	last_attrs = col & ~( BG_MASK | FG_MASK );
 }
 
 void term_move(TERM_WINDOW *window, int x, int y)
@@ -463,7 +535,7 @@ void term_clrtoeol(TERM_WINDOW *window)
 {
 	/* clrtoeol() doesn't necessarily understand colors */
 	if (last_fg == -1 && last_bg == -1 &&
-	    (last_attrs & (ATTR_UNDERLINE|ATTR_REVERSE)) == 0) {
+	    (last_attrs & (ATTR_UNDERLINE|ATTR_REVERSE|ATTR_ITALIC)) == 0) {
 		if (!term_lines_empty[vcy]) {
 			if (vcmove) term_move_real();
 			terminfo_clrtoeol();
@@ -522,7 +594,7 @@ void term_stop(void)
 
 static int input_utf8(const unsigned char *buffer, int size, unichar *result)
 {
-	unichar c = g_utf8_get_char_validated(buffer, size);
+	unichar c = g_utf8_get_char_validated((char *)buffer, size);
 
 	switch (c) {
 	case (unichar)-1:
