@@ -106,6 +106,7 @@ static void event_names_list(IRC_SERVER_REC *server, const char *data)
 	char *params, *type, *channel, *names, *ptr, *host;
         int op, halfop, voice;
 	char prefixes[MAX_USER_PREFIXES+1];
+	const char *nick_flags, *nick_flag_cur, *nick_flag_op;
 
 	g_return_if_fail(data != NULL);
 
@@ -117,6 +118,8 @@ static void event_names_list(IRC_SERVER_REC *server, const char *data)
 		g_free(params);
 		return;
 	}
+	nick_flags = server->get_nick_flags(SERVER(server));
+	nick_flag_op = strchr(nick_flags, '@');
 
 	/* type = '=' = public, '*' = private, '@' = secret.
 
@@ -158,6 +161,15 @@ static void event_names_list(IRC_SERVER_REC *server, const char *data)
 			case '+':
                                 voice = TRUE;
                                 break;
+			default:
+				/* If this flag is listed higher than op (in the
+				 * isupport PREFIX reply), then count this user
+				 * as an op. */
+				nick_flag_cur = strchr(nick_flags, *ptr);
+				if (nick_flag_cur && nick_flag_op && nick_flag_cur < nick_flag_op) {
+					op = TRUE;
+				}
+				break;
 			}
                         ptr++;
 		}
@@ -183,7 +195,7 @@ static void event_end_of_names(IRC_SERVER_REC *server, const char *data)
 	IRC_CHANNEL_REC *chanrec;
 	NICK_REC *ownnick;
 	int nicks;
-	
+
 	g_return_if_fail(server != NULL);
 
 	params = event_get_params(data, 2, NULL, &channel);
@@ -342,9 +354,9 @@ static void event_nick_in_use(IRC_SERVER_REC *server, const char *data)
 	}
 
 	/* nick already in use - need to change it .. */
-	if (g_strcasecmp(server->nick, server->connrec->nick) == 0 &&
+	if (g_ascii_strcasecmp(server->nick, server->connrec->nick) == 0 &&
 	    server->connrec->alternate_nick != NULL &&
-	    g_strcasecmp(server->connrec->alternate_nick, server->nick) != 0) {
+	    g_ascii_strcasecmp(server->connrec->alternate_nick, server->nick) != 0) {
 		/* first try, so try the alternative nick.. */
 		g_free(server->nick);
 		server->nick = g_strdup(server->connrec->alternate_nick);
@@ -372,7 +384,7 @@ static void event_nick_in_use(IRC_SERVER_REC *server, const char *data)
 
 	cmd = g_strdup_printf("NICK %s", server->nick);
 	irc_send_cmd_now(server, cmd);
-	g_free(cmd);	
+	g_free(cmd);
 }
 
 static void event_target_unavailable(IRC_SERVER_REC *server, const char *data)
@@ -400,10 +412,10 @@ static void event_nick(IRC_SERVER_REC *server, const char *data,
 
 	params = event_get_params(data, 1, &nick);
 
-	if (g_strcasecmp(orignick, server->nick) == 0) {
+	if (g_ascii_strcasecmp(orignick, server->nick) == 0) {
 		/* You changed your nick */
 		if (server->last_nick != NULL &&
-		    g_strcasecmp(server->last_nick, nick) == 0) {
+		    g_ascii_strcasecmp(server->last_nick, nick) == 0) {
                         /* changed with /NICK - keep it as wanted nick */
 			g_free(server->connrec->nick);
 			server->connrec->nick = g_strdup(nick);
