@@ -13,16 +13,14 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
 #include "module.h"
 
-#include "net-nonblock.h"
 #include "net-sendbuffer.h"
-#include "line-split.h"
 #include "signals.h"
 #include "rawlog.h"
 #include "misc.h"
@@ -30,6 +28,8 @@
 #include "channels.h"
 #include "queries.h"
 
+#include "irc-nicklist.h"
+#include "irc-queries.h"
 #include "irc-servers-setup.h"
 #include "irc-servers.h"
 #include "channel-rejoin.h"
@@ -39,6 +39,7 @@
 #include "modes.h"
 
 #include "settings.h"
+#include "recode.h"
 
 #define DEFAULT_MAX_KICKS 1
 #define DEFAULT_MAX_MODES 3
@@ -46,18 +47,26 @@
 #define DEFAULT_MAX_MSGS 1
 
 #define DEFAULT_USER_MODE "+i"
-#define DEFAULT_CMD_QUEUE_SPEED 2200
+#define DEFAULT_CMD_QUEUE_SPEED "2200msec"
 #define DEFAULT_CMDS_MAX_AT_ONCE 5
 #define DEFAULT_MAX_QUERY_CHANS 1 /* more and more IRC networks are using stupid ircds.. */
 
+/*
+ * 63 is the maximum hostname length defined by the protocol.  10 is a common
+ * username limit on many networks.  1 is for the `@'.
+ */
+#define MAX_USERHOST_LEN (63 + 10 + 1)
+
 void irc_servers_reconnect_init(void);
 void irc_servers_reconnect_deinit(void);
 
 static int cmd_tag;
 
-static int isnickflag_func(char flag)
+static int isnickflag_func(SERVER_REC *server, char flag)
 {
-	return isnickflag(flag);
+	IRC_SERVER_REC *irc_server = (IRC_SERVER_REC *) server;
+
+	return isnickflag(irc_server, flag);
 }
 
 static int ischannel_func(SERVER_REC *server, const char *data)
@@ -71,12 +80,79 @@ static int ischannel_func(SERVER_REC *server, const char *data)
 	return ischannel(*data);
 }
 
+static char **split_line(const SERVER_REC *server, const char *line,
+			 const char *target, int len)
+{
+	const char *start = settings_get_str("split_line_start");
+	const char *end = settings_get_str("split_line_end");
+	char *recoded_start = recode_out(server, start, target);
+	char *recoded_end = recode_out(server, end, target);
+	char **lines;
+	int i;
+
+	/*
+	 * Having the same length limit on all lines will make the first line
+	 * shorter than necessary if `split_line_start' is set, but it makes
+	 * the code much simpler.  It's worth it.
+	 */
+	len -= strlen(recoded_start) + strlen(recoded_end);
+	if (len <= 0) {
+		/* There is no room for anything. */
+		g_free(recoded_start);
+		g_free(recoded_end);
+		return NULL;
+	}
+
+	lines = recode_split(server, line, target, len);
+	for (i = 0; lines[i] != NULL; i++) {
+		if (i != 0 && *start != '\0') {
+			/* Not the first line. */
+			char *tmp = lines[i];
+			lines[i] = g_strconcat(start, tmp, NULL);
+			g_free(tmp);
+		}
+		if (lines[i + 1] != NULL && *end != '\0') {
+			/* Not the last line. */
+			char *tmp = lines[i];
+
+			if (lines[i + 2] == NULL) {
+				/* Next to last line.  Check if we have room
+				 * to append the last line to the current line,
+				 * to avoid an unnecessary line break.
+				 */
+				char *recoded_l = recode_out(server,
+							     lines[i+1],
+							     target);
+				if (strlen(recoded_l) <= strlen(recoded_end)) {
+					lines[i] = g_strconcat(tmp, lines[i+1],
+							       NULL);
+					g_free_and_null(lines[i+1]);
+					lines = g_renew(char *, lines, i + 2);
+
+					g_free(recoded_l);
+					g_free(tmp);
+					break;
+				}
+				g_free(recoded_l);
+			}
+
+			lines[i] = g_strconcat(tmp, end, NULL);
+			g_free(tmp);
+		}
+	}
+
+	g_free(recoded_start);
+	g_free(recoded_end);
+	return lines;
+}
+
 static void send_message(SERVER_REC *server, const char *target,
 			 const char *msg, int target_type)
 {
 	IRC_SERVER_REC *ircserver;
 	CHANNEL_REC *channel;
 	char *str;
+	char *recoded;
 
         ircserver = IRC_SERVER(server);
 	g_return_if_fail(ircserver != NULL);
@@ -86,76 +162,109 @@ static void send_message(SERVER_REC *server, const char *target,
 	if (*target == '!') {
 		/* !chan -> !12345chan */
 		channel = channel_find(server, target);
-		if (channel != NULL && g_strcasecmp(channel->name, target) != 0)
+		if (channel != NULL &&
+		    g_ascii_strcasecmp(channel->name, target) != 0)
 			target = channel->name;
 	}
 
-	str = g_strdup_printf("PRIVMSG %s :%s", target, msg);
+	recoded = recode_out(SERVER(server), msg, target);
+	str = g_strdup_printf("PRIVMSG %s :%s", target, recoded);
 	irc_send_cmd_split(ircserver, str, 2, ircserver->max_msgs_in_cmd);
 	g_free(str);
+	g_free(recoded);
+}
+
+static char **split_message(SERVER_REC *server, const char *target,
+			    const char *msg)
+{
+	IRC_SERVER_REC *ircserver = IRC_SERVER(server);
+
+	g_return_val_if_fail(ircserver != NULL, NULL);
+	g_return_val_if_fail(target != NULL, NULL);
+	g_return_val_if_fail(msg != NULL, NULL);
+
+	/* length calculation shamelessly stolen from splitlong_safe.pl */
+	return split_line(SERVER(server), msg, target,
+			  510 - strlen(":! PRIVMSG  :") -
+			  strlen(ircserver->nick) - MAX_USERHOST_LEN -
+			  strlen(target));
 }
 
 static void server_init(IRC_SERVER_REC *server)
 {
 	IRC_SERVER_CONNECT_REC *conn;
-	char hostname[100], *address, *ptr, *username;
+	char *address, *ptr, *username, *cmd;
+	GTimeVal now;
 
 	g_return_if_fail(server != NULL);
 
 	conn = server->connrec;
 
 	if (conn->proxy != NULL && conn->proxy_password != NULL &&
-	    *conn->proxy_password != '\0')
-		irc_send_cmdv(server, "PASS %s", conn->proxy_password);
+	    *conn->proxy_password != '\0') {
+		cmd = g_strdup_printf("PASS %s", conn->proxy_password);
+		irc_send_cmd_now(server, cmd);
+		g_free(cmd);
+	}
 
-	if (conn->proxy != NULL && conn->proxy_string != NULL)
-		irc_send_cmdv(server, conn->proxy_string, conn->address, conn->port);
+	if (conn->proxy != NULL && conn->proxy_string != NULL) {
+		cmd = g_strdup_printf(conn->proxy_string, conn->address, conn->port);
+		irc_send_cmd_now(server, cmd);
+		g_free(cmd);
+	}
 
 	if (conn->password != NULL && *conn->password != '\0') {
                 /* send password */
-		server->cmdcount = 0;
-		irc_send_cmdv(server, "PASS %s", conn->password);
+		cmd = g_strdup_printf("PASS %s", conn->password);
+		irc_send_cmd_now(server, cmd);
+		g_free(cmd);
 	}
 
         /* send nick */
-	server->cmdcount = 0;
-	irc_send_cmdv(server, "NICK %s", conn->nick);
+	cmd = g_strdup_printf("NICK %s", conn->nick);
+	irc_send_cmd_now(server, cmd);
+	g_free(cmd);
 
 	/* send user/realname */
-	server->cmdcount = 0;
-
-	if (gethostname(hostname, sizeof(hostname)) != 0 || *hostname == '\0')
-		strcpy(hostname, "xx");
-
 	address = server->connrec->address;
         ptr = strrchr(address, ':');
 	if (ptr != NULL) {
 		/* IPv6 address .. doesn't work here, use the string after
 		   the last : char */
-                address = ptr+1;
-	}
-
-	/* Replace ':' with '_' in our own hostname (the same IPv6 problem) */
-	for (ptr = hostname; *ptr != '\0'; ptr++) {
-		if (*ptr == ':') *ptr = '_';
+		address = ptr+1;
+		if (*address == '\0')
+			address = "x";
 	}
 
 	username = g_strdup(conn->username);
 	ptr = strchr(username, ' ');
 	if (ptr != NULL) *ptr = '\0';
 
-	irc_send_cmdv(server, "USER %s %s %s :%s", username, hostname,
-		      address, conn->realname);
+	cmd = g_strdup_printf("USER %s %s %s :%s", username, username, address, conn->realname);
+	irc_send_cmd_now(server, cmd);
+	g_free(cmd);
 	g_free(username);
 
-	server->cmdcount = 0;
-
 	if (conn->proxy != NULL && conn->proxy_string_after != NULL) {
-		irc_send_cmdv(server, conn->proxy_string_after,
-			      conn->address, conn->port);
+		cmd = g_strdup_printf(conn->proxy_string_after, conn->address, conn->port);
+		irc_send_cmd_now(server, cmd);
+		g_free(cmd);
 	}
 
+	server->isupport = g_hash_table_new((GHashFunc) g_istr_hash,
+					    (GCompareFunc) g_istr_equal);
+
+	/* set the standards */
+	g_hash_table_insert(server->isupport, g_strdup("CHANMODES"), g_strdup("beI,k,l,imnpst"));
+	g_hash_table_insert(server->isupport, g_strdup("PREFIX"), g_strdup("(ohv)@%+"));
+
 	server->cmdcount = 0;
+
+	/* prevent the queue from sending too early, we have a max cut off of 120 secs */
+	/* this will reset to 1 sec after we get the 001 event */
+	g_get_current_time(&now);
+	memcpy(&((IRC_SERVER_REC *)server)->wait_cmd, &now, sizeof(GTimeVal));
+	((IRC_SERVER_REC *)server)->wait_cmd.tv_sec += 120;
 }
 
 SERVER_REC *irc_server_init_connect(SERVER_CONNECT_REC *conn)
@@ -176,11 +285,11 @@ SERVER_REC *irc_server_init_connect(SERVER_CONNECT_REC *conn)
 
 	if (server->connrec->port <= 0) {
 		server->connrec->port =
-			server->connrec->use_ssl ? 9999 : 6667;
+			server->connrec->use_ssl ? 6697 : 6667;
 	}
 
 	server->cmd_queue_speed = ircconn->cmd_queue_speed > 0 ?
-		ircconn->cmd_queue_speed : settings_get_int("cmd_queue_speed");
+		ircconn->cmd_queue_speed : settings_get_time("cmd_queue_speed");
 	server->max_cmds_at_once = ircconn->max_cmds_at_once > 0 ?
 		ircconn->max_cmds_at_once : settings_get_int("cmds_max_at_once");
 	server->max_query_chans = ircconn->max_query_chans > 0 ?
@@ -196,6 +305,8 @@ SERVER_REC *irc_server_init_connect(SERVER_CONNECT_REC *conn)
 		ircconn->max_msgs : DEFAULT_MAX_MSGS;
 	server->connrec->use_ssl = conn->use_ssl;
 
+	modes_server_init(server);
+
         server_connect_init((SERVER_REC *) server);
 	return (SERVER_REC *) server;
 }
@@ -239,7 +350,7 @@ void irc_server_purge_output(IRC_SERVER_REC *server, const char *target)
                 redirect = tmp->next->data;
 
 		if ((target == NULL || command_has_target(cmd, target)) &&
-		    g_strncasecmp(cmd, "PONG ", 5) != 0) {
+		    g_ascii_strncasecmp(cmd, "PONG ", 5) != 0) {
                         /* remove the redirection */
                         link = tmp->next;
 			server->cmdqueue =
@@ -265,7 +376,11 @@ static void sig_connected(IRC_SERVER_REC *server)
 
 	server->isnickflag = isnickflag_func;
 	server->ischannel = ischannel_func;
+	server->split_message = split_message;
 	server->send_message = send_message;
+	server->query_find_func =
+		(QUERY_REC *(*)(SERVER_REC *, const char *)) irc_query_find;
+	server->nick_comp_func = irc_nickcmp_rfc1459;
 
 	server->splits = g_hash_table_new((GHashFunc) g_istr_hash,
 					  (GCompareFunc) g_istr_equal);
@@ -274,6 +389,12 @@ static void sig_connected(IRC_SERVER_REC *server)
 		server_init(server);
 }
 
+static void isupport_destroy_hash(void *key, void *value)
+{
+	g_free(key);
+	g_free(value);
+}
+
 static void sig_disconnected(IRC_SERVER_REC *server)
 {
 	GSList *tmp;
@@ -289,6 +410,12 @@ static void sig_disconnected(IRC_SERVER_REC *server)
 	g_slist_free(server->cmdqueue);
         server->cmdqueue = NULL;
 
+	/* these are dynamically allocated only if isupport was sent */
+	g_hash_table_foreach(server->isupport,
+			     (GHFunc) isupport_destroy_hash, server);
+	g_hash_table_destroy(server->isupport);
+	server->isupport = NULL;
+
 	g_free_and_null(server->wanted_usermode);
 	g_free_and_null(server->real_address);
 	g_free_and_null(server->usermode);
@@ -299,14 +426,59 @@ static void sig_disconnected(IRC_SERVER_REC *server)
 static void sig_server_quit(IRC_SERVER_REC *server, const char *msg)
 {
 	char *str;
+	char *recoded;
 
-	if (!IS_IRC_SERVER(server) || server->handle == NULL ||
-	    server->buffer == NULL)
+	if (!IS_IRC_SERVER(server) || !server->connected)
 		return;
 
-	str = g_strdup_printf("QUIT :%s", msg);
+	recoded = recode_out(SERVER(server), msg, NULL);
+	str = g_strdup_printf("QUIT :%s", recoded);
 	irc_send_cmd_now(server, str);
 	g_free(str);
+	g_free(recoded);
+}
+
+void irc_server_send_action(IRC_SERVER_REC *server, const char *target, const char *data)
+{
+	char *recoded;
+
+	recoded = recode_out(SERVER(server), data, target);
+	irc_send_cmdv(server, "PRIVMSG %s :\001ACTION %s\001", target, recoded);
+	g_free(recoded);
+}
+
+char **irc_server_split_action(IRC_SERVER_REC *server, const char *target,
+			       const char *data)
+{
+	g_return_val_if_fail(server != NULL, NULL);
+	g_return_val_if_fail(target != NULL, NULL);
+	g_return_val_if_fail(data != NULL, NULL);
+
+	return split_line(SERVER(server), data, target,
+			  510 - strlen(":! PRIVMSG  :\001ACTION \001") -
+			  strlen(server->nick) - MAX_USERHOST_LEN -
+			  strlen(target));
+}
+
+void irc_server_send_away(IRC_SERVER_REC *server, const char *reason)
+{
+	char *recoded = NULL;
+
+	if (!IS_IRC_SERVER(server))
+		return;
+
+	if (*reason != '\0' || server->usermode_away) {
+		g_free_and_null(server->away_reason);
+                if (*reason != '\0') {
+			server->away_reason = g_strdup(reason);
+			reason = recoded = recode_out(SERVER(server), reason, NULL);
+			irc_send_cmdv(server, "AWAY :%s", reason);
+		} else {
+			irc_send_cmdv(server, "AWAY");
+		}
+
+	}
+	g_free(recoded);
 }
 
 void irc_server_send_data(IRC_SERVER_REC *server, const char *data, int len)
@@ -323,7 +495,7 @@ void irc_server_send_data(IRC_SERVER_REC *server, const char *data, int len)
 	   actually is 1sec / 100 bytes penalty, but we rather want to deal
 	   with the max. 1000 bytes input buffer problem. If we send more
 	   than that with the burst, we'll get excess flooded. */
-	if (len < 100)
+	if (len < 100 || server->cmd_queue_speed <= 10)
 		server->wait_cmd.tv_sec = 0;
 	else {
 		memcpy(&server->wait_cmd, &server->last_cmd, sizeof(GTimeVal));
@@ -331,7 +503,7 @@ void irc_server_send_data(IRC_SERVER_REC *server, const char *data, int len)
 	}
 }
 
-static void server_cmd_timeout(IRC_SERVER_REC *server, GTimeVal *now)
+static int server_cmd_timeout(IRC_SERVER_REC *server, GTimeVal *now)
 {
 	REDIRECT_REC *redirect;
         GSList *link;
@@ -340,20 +512,20 @@ static void server_cmd_timeout(IRC_SERVER_REC *server, GTimeVal *now)
 	int len;
 
 	if (!IS_IRC_SERVER(server))
-		return;
+		return 0;
 
 	if (server->cmdcount == 0 && server->cmdqueue == NULL)
-		return;
+		return 0;
 
 	if (g_timeval_cmp(now, &server->wait_cmd) == -1)
-		return;
+		return 1;
 
 	usecs = get_timeval_diff(now, &server->last_cmd);
 	if (usecs < server->cmd_queue_speed)
-		return;
+		return 1;
 
 	server->cmdcount--;
-	if (server->cmdqueue == NULL) return;
+	if (server->cmdqueue == NULL) return 1;
 
         /* get command */
 	cmd = server->cmdqueue->data;
@@ -363,8 +535,7 @@ static void server_cmd_timeout(IRC_SERVER_REC *server, GTimeVal *now)
 	len = strlen(cmd);
 	irc_server_send_data(server, cmd, len);
 
-	/* add to rawlog without [CR+]LF (/RAWQUOTE might not have
-	   added the CR) */
+	/* add to rawlog without [CR+]LF */
         if (len > 2 && cmd[len-2] == '\r')
 		cmd[len-2] = '\0';
         else if (cmd[len-1] == '\n')
@@ -373,22 +544,39 @@ static void server_cmd_timeout(IRC_SERVER_REC *server, GTimeVal *now)
 	server_redirect_command(server, cmd, redirect);
 
 	/* remove from queue */
-	g_free(cmd);
 	server->cmdqueue = g_slist_remove(server->cmdqueue, cmd);
+	g_free(cmd);
 
         link = server->cmdqueue;
 	server->cmdqueue = g_slist_remove_link(server->cmdqueue, link);
         g_slist_free_1(link);
+	return 1;
 }
 
 /* check every now and then if there's data to be sent in command buffer */
 static int servers_cmd_timeout(void)
 {
 	GTimeVal now;
+	GSList *tmp;
+	int keep = 0;
 
 	g_get_current_time(&now);
-	g_slist_foreach(servers, (GFunc) server_cmd_timeout, &now);
-	return 1;
+	for (tmp = servers; tmp != NULL; tmp = tmp->next) {
+		keep |= server_cmd_timeout(tmp->data, &now);
+	}
+	if (keep)
+		return 1;
+	else {
+		cmd_tag = -1;
+		return 0;
+	}
+}
+
+/* Start the timeout for sending data later and decreasing cmdcount again */
+void irc_servers_start_cmd_timeout(void)
+{
+	if (cmd_tag == -1)
+		cmd_tag = g_timeout_add(500, (GSourceFunc) servers_cmd_timeout, NULL);
 }
 
 /* Return a string of all channels (and keys, if any have them) in server,
@@ -410,8 +598,8 @@ char *irc_server_get_channels(IRC_SERVER_REC *server)
 	for (tmp = server->channels; tmp != NULL; tmp = tmp->next) {
 		CHANNEL_REC *channel = tmp->data;
 
-		g_string_sprintfa(chans, "%s,", channel->name);
-		g_string_sprintfa(keys, "%s,", channel->key == NULL ? "x" :
+		g_string_append_printf(chans, "%s,", channel->name);
+		g_string_append_printf(keys, "%s,", channel->key == NULL ? "x" :
 				  channel->key);
 		if (channel->key != NULL)
 			use_keys = TRUE;
@@ -421,8 +609,8 @@ char *irc_server_get_channels(IRC_SERVER_REC *server)
 	for (tmp = server->rejoin_channels; tmp != NULL; tmp = tmp->next) {
 		REJOIN_REC *rec = tmp->data;
 
-		g_string_sprintfa(chans, "%s,", rec->channel);
-		g_string_sprintfa(keys, "%s,", rec->key == NULL ? "x" :
+		g_string_append_printf(chans, "%s,", rec->channel);
+		g_string_append_printf(keys, "%s,", rec->key == NULL ? "x" :
 				  rec->key);
 		if (rec->key != NULL) use_keys = TRUE;
 	}
@@ -430,7 +618,7 @@ char *irc_server_get_channels(IRC_SERVER_REC *server)
 	if (chans->len > 0) {
 		g_string_truncate(chans, chans->len-1);
 		g_string_truncate(keys, keys->len-1);
-		if (use_keys) g_string_sprintfa(chans, " %s", keys->str);
+		if (use_keys) g_string_append_printf(chans, " %s", keys->str);
 	}
 
 	ret = chans->str;
@@ -440,36 +628,10 @@ char *irc_server_get_channels(IRC_SERVER_REC *server)
 	return ret;
 }
 
-static int sig_set_user_mode(IRC_SERVER_REC *server)
-{
-	const char *mode;
-	char *newmode, *args;
-
-	if (g_slist_find(servers, server) == NULL)
-		return 0; /* got disconnected */
-
-	mode = server->connrec->usermode;
-	newmode = server->usermode == NULL ? NULL :
-		modes_join(server->usermode, mode, FALSE);
-
-	if (newmode == NULL || strcmp(newmode, server->usermode) != 0) {
-		/* change the user mode. we used to do some trickery to
-		   get rid of unwanted modes at reconnect time, but that's
-		   more trouble than worth. (eg. we don't want to remove
-		   some good default server modes, but we don't want to
-		   set back +r, etc..) */
-		args = g_strdup_printf("%s %s", server->nick, mode);
-		signal_emit("command mode", 3, args, server, NULL);
-		g_free(args);
-	}
-
-	g_free_not_null(newmode);
-	return 0;
-}
-
 static void event_connected(IRC_SERVER_REC *server, const char *data, const char *from)
 {
 	char *params, *nick;
+	GTimeVal now;
 
 	g_return_if_fail(server != NULL);
 
@@ -491,9 +653,20 @@ static void event_connected(IRC_SERVER_REC *server, const char *data, const char
 	server->connected = 1;
 	server->real_connect_time = time(NULL);
 
+	/* let the queue send now that we are identified */
+	g_get_current_time(&now);
+	memcpy(&server->wait_cmd, &now, sizeof(GTimeVal));
+
 	if (server->connrec->usermode != NULL) {
-		/* wait a second and then send the user mode */
-		g_timeout_add(1000, (GSourceFunc) sig_set_user_mode, server);
+		/* Send the user mode, before the autosendcmd.
+		 * Do not pass this through cmd_mode because it
+		 * is not known whether the resulting MODE message
+		 * (if any) is the initial umode or a reply to this.
+		 */
+		irc_send_cmdv(server, "MODE %s %s", server->nick,
+				server->connrec->usermode);
+		g_free_not_null(server->wanted_usermode);
+		server->wanted_usermode = g_strdup(server->connrec->usermode);
 	}
 
 	signal_emit("event connected", 1, server);
@@ -519,6 +692,109 @@ static void event_server_info(IRC_SERVER_REC *server, const char *data)
 	g_free(params);
 }
 
+static void parse_chanmodes(IRC_SERVER_REC *server, const char *sptr)
+{
+	mode_func_t *modefuncs[] = {
+		modes_type_a,
+		modes_type_b,
+		modes_type_c,
+		modes_type_d
+	};
+	char **item, **chanmodes;
+	int i;
+
+	chanmodes = g_strsplit(sptr, ",", 5); /* ignore extras */
+
+	for (i = 0, item = chanmodes; *item != NULL && i < 4; item++, i++) {
+		unsigned char *p = (unsigned char*) *item;
+		while (*p != '\0') {
+			server->modes[(int)*p].func = modefuncs[i];
+			p++;
+		}
+	}
+
+	g_strfreev(chanmodes);
+}
+
+static void parse_prefix(IRC_SERVER_REC *server, const char *sptr)
+{
+	const char *eptr;
+
+	if (*sptr++ != '(')
+		return; /* Unknown prefix format */
+
+	eptr = strchr(sptr, ')');
+	if (eptr == NULL)
+		return;
+
+	eptr++;
+	while (*sptr != '\0' && *eptr != '\0' && *sptr != ')' && *eptr != ' ') {
+		server->modes[(int)(unsigned char) *sptr].func =
+			modes_type_prefix;
+		server->modes[(int)(unsigned char) *sptr].prefix = *eptr;
+		server->prefix[(int)(unsigned char) *eptr] = *sptr;
+		sptr++; eptr++;
+	}
+}
+
+
+static void event_isupport(IRC_SERVER_REC *server, const char *data)
+{
+	char **item, *sptr, *eptr;
+	char **isupport;
+	gpointer key, value;
+
+	g_return_if_fail(server != NULL);
+
+	server->isupport_sent = TRUE;
+
+	sptr = strchr(data, ' ');
+	if (sptr == NULL)
+		return;
+	sptr++;
+
+	isupport = g_strsplit(sptr, " ", -1);
+
+	for(item = isupport; *item != NULL; item++) {
+		int removed = FALSE;
+
+		if (**item == '\0')
+			continue;
+
+		if (**item == ':')
+			break;
+
+		sptr = strchr(*item, '=');
+		if (sptr != NULL) {
+			*sptr = '\0';
+			sptr++;
+		}
+
+		eptr = *item;
+		if(*eptr == '-') {
+			removed = TRUE;
+			eptr++;
+		}
+
+		key = value = NULL;
+		if (!g_hash_table_lookup_extended(server->isupport, eptr,
+						  &key, &value) && removed)
+			continue;
+
+		g_hash_table_remove(server->isupport, eptr);
+		if (!removed) {
+			g_hash_table_insert(server->isupport, g_strdup(eptr),
+					    g_strdup(sptr != NULL ? sptr : ""));
+		}
+
+		g_free(key);
+		g_free(value);
+	}
+	g_strfreev(isupport);
+	irc_server_init_isupport(server);
+
+}
+
 static void event_motd(IRC_SERVER_REC *server, const char *data, const char *from)
 {
 	if (server->connected)
@@ -552,6 +828,39 @@ static void event_channels_formed(IRC_SERVER_REC *server, const char *data)
 	g_free(params);
 }
 
+static void event_hosthidden(IRC_SERVER_REC *server, const char *data)
+{
+	char *params, *newhost, *p, *newuserhost;
+
+	g_return_if_fail(server != NULL);
+
+	params = event_get_params(data, 2, NULL, &newhost);
+	/* do a sanity check */
+	if (!strchr(newhost, '*') && !strchr(newhost, '?') &&
+			!strchr(newhost, '!') && !strchr(newhost, '#') &&
+			!strchr(newhost, '&') && !strchr(newhost, ' ') &&
+			*newhost != '\0' && *newhost != '@' &&
+			*newhost != ':' && *newhost != '-' &&
+			newhost[strlen(newhost) - 1] != '-') {
+		if (strchr(newhost, '@')) {
+			newuserhost = g_strdup(newhost);
+			g_free(server->userhost);
+			server->userhost = newuserhost;
+		} else if (server->userhost != NULL) {
+			/* no user@, only process if we know the user@
+			 * already
+			 */
+			p = strchr(server->userhost, '@');
+			if (p == NULL)
+				p = server->userhost;
+			newuserhost = g_strdup_printf("%.*s@%s", (int)(p - server->userhost), server->userhost, newhost);
+			g_free(server->userhost);
+			server->userhost = newuserhost;
+		}
+	}
+	g_free(params);
+}
+
 static void event_server_banned(IRC_SERVER_REC *server, const char *data)
 {
 	g_return_if_fail(server != NULL);
@@ -564,7 +873,9 @@ static void event_error(IRC_SERVER_REC *server, const char *data)
 	g_return_if_fail(server != NULL);
 
 	if (!server->connected && (stristr(data, "Unauthorized") != NULL ||
-				   stristr(data, "K-lined") != NULL))
+				   stristr(data, "K-lined") != NULL ||
+				   stristr(data, "Banned") != NULL ||
+				   stristr(data, "Bad user info") != NULL))
 		server->banned = TRUE;
 }
 
@@ -584,23 +895,99 @@ static void event_empty(void)
 {
 }
 
+void irc_server_init_isupport(IRC_SERVER_REC *server)
+{
+	char *sptr;
+	gpointer key, value;
+	/* chanmodes/prefix will fully override defaults */
+	memset(server->modes, 0, sizeof(server->modes));
+	memset(server->prefix, 0, sizeof(server->prefix));
+
+	if ((sptr = g_hash_table_lookup(server->isupport, "CHANMODES")))
+		parse_chanmodes(server, sptr);
+
+	/* This is after chanmode because some servers define modes in both */
+	if (g_hash_table_lookup_extended(server->isupport, "PREFIX",
+					 &key, &value)) {
+		sptr = value;
+		if (*sptr != '(') {
+			/* server incompatible with isupport draft */
+			g_hash_table_remove(server->isupport, key);
+			g_free(key);
+			g_free(value);
+			sptr = NULL;
+		}
+	} else {
+		sptr = NULL;
+	}
+
+	if (sptr == NULL) {
+		sptr = g_strdup("(ohv)@%+");
+		g_hash_table_insert(server->isupport, g_strdup("PREFIX"), sptr);
+	}
+	parse_prefix(server, sptr);
+
+	if ((sptr = g_hash_table_lookup(server->isupport, "MODES"))) {
+		server->max_modes_in_cmd = atoi(sptr);
+		if (server->max_modes_in_cmd < 1)
+			server->max_modes_in_cmd = DEFAULT_MAX_MODES;
+	}
+
+	if ((sptr = g_hash_table_lookup(server->isupport, "CASEMAPPING"))) {
+		if (strstr(sptr, "rfc1459") != NULL)
+			server->nick_comp_func = irc_nickcmp_rfc1459;
+		else
+			server->nick_comp_func = irc_nickcmp_ascii;
+	}
+
+	if ((sptr = g_hash_table_lookup(server->isupport, "TARGMAX"))) {
+		char *p = sptr;
+		server->max_kicks_in_cmd = 1;
+		server->max_msgs_in_cmd = 1;
+		/* Not doing WHOIS here until it is clear what it means. */
+		while (*p != '\0') {
+			if (!g_ascii_strncasecmp(p, "KICK:", 5)) {
+				server->max_kicks_in_cmd = atoi(p + 5);
+				if (server->max_kicks_in_cmd <= 0)
+					server->max_kicks_in_cmd = 30;
+			} else if (!g_ascii_strncasecmp(p, "PRIVMSG:", 8)) {
+				server->max_msgs_in_cmd = atoi(p + 8);
+				if (server->max_msgs_in_cmd <= 0)
+					server->max_msgs_in_cmd = 30;
+			}
+			p = strchr(p, ',');
+			if (p == NULL)
+				break;
+			p++;
+		}
+	} else if ((sptr = g_hash_table_lookup(server->isupport, "MAXTARGETS"))) {
+		server->max_msgs_in_cmd = atoi(sptr);
+		if (server->max_msgs_in_cmd <= 0)
+			server->max_msgs_in_cmd = 1;
+	}
+}
+
 void irc_servers_init(void)
 {
 	settings_add_str("misc", "usermode", DEFAULT_USER_MODE);
-	settings_add_int("flood", "cmd_queue_speed", DEFAULT_CMD_QUEUE_SPEED);
+	settings_add_str("misc", "split_line_start", "");
+	settings_add_str("misc", "split_line_end", "");
+	settings_add_time("flood", "cmd_queue_speed", DEFAULT_CMD_QUEUE_SPEED);
 	settings_add_int("flood", "cmds_max_at_once", DEFAULT_CMDS_MAX_AT_ONCE);
 
-	cmd_tag = g_timeout_add(500, (GSourceFunc) servers_cmd_timeout, NULL);
+	cmd_tag = -1;
 
 	signal_add_first("server connected", (SIGNAL_FUNC) sig_connected);
 	signal_add_last("server disconnected", (SIGNAL_FUNC) sig_disconnected);
 	signal_add_last("server quit", (SIGNAL_FUNC) sig_server_quit);
 	signal_add("event 001", (SIGNAL_FUNC) event_connected);
 	signal_add("event 004", (SIGNAL_FUNC) event_server_info);
+	signal_add("event 005", (SIGNAL_FUNC) event_isupport);
 	signal_add("event 375", (SIGNAL_FUNC) event_motd);
 	signal_add_last("event 376", (SIGNAL_FUNC) event_end_of_motd);
 	signal_add_last("event 422", (SIGNAL_FUNC) event_end_of_motd); /* no motd */
 	signal_add("event 254", (SIGNAL_FUNC) event_channels_formed);
+	signal_add("event 396", (SIGNAL_FUNC) event_hosthidden);
 	signal_add("event 465", (SIGNAL_FUNC) event_server_banned);
 	signal_add("event error", (SIGNAL_FUNC) event_error);
 	signal_add("event ping", (SIGNAL_FUNC) event_ping);
@@ -614,17 +1001,20 @@ void irc_servers_init(void)
 
 void irc_servers_deinit(void)
 {
-	g_source_remove(cmd_tag);
+	if (cmd_tag != -1)
+		g_source_remove(cmd_tag);
 
 	signal_remove("server connected", (SIGNAL_FUNC) sig_connected);
 	signal_remove("server disconnected", (SIGNAL_FUNC) sig_disconnected);
         signal_remove("server quit", (SIGNAL_FUNC) sig_server_quit);
 	signal_remove("event 001", (SIGNAL_FUNC) event_connected);
 	signal_remove("event 004", (SIGNAL_FUNC) event_server_info);
+	signal_remove("event 005", (SIGNAL_FUNC) event_isupport);
 	signal_remove("event 375", (SIGNAL_FUNC) event_motd);
 	signal_remove("event 376", (SIGNAL_FUNC) event_end_of_motd);
 	signal_remove("event 422", (SIGNAL_FUNC) event_end_of_motd); /* no motd */
 	signal_remove("event 254", (SIGNAL_FUNC) event_channels_formed);
+	signal_remove("event 396", (SIGNAL_FUNC) event_hosthidden);
 	signal_remove("event 465", (SIGNAL_FUNC) event_server_banned);
 	signal_remove("event error", (SIGNAL_FUNC) event_error);
 	signal_remove("event ping", (SIGNAL_FUNC) event_ping);
