@@ -23,9 +23,7 @@
 #include "commands.h"
 #include "recode.h"
 #include "network.h"
-#include "net-nonblock.h"
 #include "net-sendbuffer.h"
-#include "line-split.h"
 #include "misc.h"
 #include "settings.h"
 
@@ -92,7 +90,6 @@ static void sig_dcc_destroyed(CHAT_DCC_REC *dcc)
 	dcc_remove_chat_refs(dcc);
 
 	if (dcc->sendbuf != NULL) net_sendbuffer_destroy(dcc->sendbuf, FALSE);
-	line_split_free(dcc->readbuf);
 	g_free(dcc->id);
 }
 
@@ -106,7 +103,7 @@ CHAT_DCC_REC *dcc_chat_find_id(const char *id)
 		CHAT_DCC_REC *dcc = tmp->data;
 
 		if (IS_DCC_CHAT(dcc) && dcc->id != NULL &&
-		    g_strcasecmp(dcc->id, id) == 0)
+		    g_ascii_strcasecmp(dcc->id, id) == 0)
 			return dcc;
 	}
 
@@ -124,7 +121,7 @@ static CHAT_DCC_REC *dcc_chat_find_nick(IRC_SERVER_REC *server,
 		CHAT_DCC_REC *dcc = tmp->data;
 
 		if (IS_DCC_CHAT(dcc) && dcc->server == server &&
-		    g_strcasecmp(dcc->nick, nick) == 0)
+		    g_ascii_strcasecmp(dcc->nick, nick) == 0)
 			return dcc;
 	}
 
@@ -284,7 +281,7 @@ static void cmd_ctcp(const char *data, IRC_SERVER_REC *server)
 
 	dcc = dcc_chat_find_id(target+1);
 	if (dcc != NULL) {
-		g_strup(ctcpcmd);
+		ascii_strup(ctcpcmd);
 
 		str = g_strconcat(ctcpcmd, " ", ctcpdata, NULL);
 		dcc_ctcp_message(server, dcc->nick, dcc, FALSE, str);
@@ -298,15 +295,14 @@ static void cmd_ctcp(const char *data, IRC_SERVER_REC *server)
 /* input function: DCC CHAT received some data.. */
 void dcc_chat_input(CHAT_DCC_REC *dcc)
 {
-        char tmpbuf[512], *str;
-	int recvlen, ret;
+	char *str;
+	int ret;
 
 	g_return_if_fail(IS_DCC_CHAT(dcc));
 
 	do {
-		recvlen = net_receive(dcc->handle, tmpbuf, sizeof(tmpbuf));
+		ret = net_sendbuffer_receive_line(dcc->sendbuf, &str, 1);
 
-		ret = line_split(tmpbuf, recvlen, &str, &dcc->readbuf);
 		if (ret == -1) {
 			/* connection lost */
                         dcc->connection_lost = TRUE;
@@ -432,7 +428,7 @@ static void dcc_chat_passive(CHAT_DCC_REC *dcc)
 	dcc_ip2str(&own_ip, host);
 	irc_send_cmdv(dcc->server, "PRIVMSG %s :\001DCC CHAT CHAT %s %d %d\001",
 		      dcc->nick, host, port, dcc->pasv_id);
-         
+
 }
 
 /* SYNTAX: DCC CHAT [-passive] [<nick>] */
@@ -540,7 +536,7 @@ static void cmd_mircdcc(const char *data, SERVER_REC *server,
 	if (dcc == NULL) return;
 
 	dcc->mirc_ctcp = i_toupper(*data) != 'N' &&
-		g_strncasecmp(data, "OF", 2) != 0;
+		g_ascii_strncasecmp(data, "OF", 2) != 0;
 }
 
 /* DCC CLOSE CHAT <nick> - check only from chat_ids in open DCC chats,
@@ -554,7 +550,7 @@ static void cmd_dcc_close(char *data, SERVER_REC *server)
 
 	g_return_if_fail(data != NULL);
 
-	if (g_strncasecmp(data, "CHAT ", 5) != 0 ||
+	if (g_ascii_strncasecmp(data, "CHAT ", 5) != 0 ||
 	    !cmd_get_params(data, &free_arg, 2, NULL, &nick))
 		return;
 
@@ -566,7 +562,7 @@ static void cmd_dcc_close(char *data, SERVER_REC *server)
 
 		next = tmp->next;
 		if (IS_DCC_CHAT(dcc) && dcc->id != NULL &&
-		    g_strcasecmp(dcc->id, nick) == 0) {
+		    g_ascii_strcasecmp(dcc->id, nick) == 0) {
 			found = TRUE;
 			if (!dcc_is_connected(dcc) && IS_IRC_SERVER(server))
 				dcc_reject(DCC(dcc), IRC_SERVER(server));
@@ -619,7 +615,7 @@ static void ctcp_msg_dcc_chat(IRC_SERVER_REC *server, const char *data,
 	char **params;
 	int paramcount;
         int passive, autoallow = FALSE;
-	
+
         /* CHAT <unused> <address> <port> */
 	/* CHAT <unused> <address> 0 <id> (DCC CHAT passive protocol) */
 	params = g_strsplit(data, " ", -1);
@@ -660,14 +656,14 @@ static void ctcp_msg_dcc_chat(IRC_SERVER_REC *server, const char *data,
 			}
 		}
 	}
-	
+
 	dcc = dcc_chat_create(server, chat, nick, params[0]);
 	dcc->target = g_strdup(target);
 	dcc->port = atoi(params[2]);
-	
+
 	if (passive)
 		dcc->pasv_id = atoi(params[3]);
-	
+
 	dcc_str2ip(params[1], &dcc->addr);
 	net_ip2host(&dcc->addr, dcc->addrstr);
 
@@ -695,11 +691,11 @@ static void dcc_chat_msg(CHAT_DCC_REC *dcc, const char *msg)
 	g_return_if_fail(msg != NULL);
 
 	reply = FALSE;
-	if (g_strncasecmp(msg, "CTCP_MESSAGE ", 13) == 0) {
+	if (g_ascii_strncasecmp(msg, "CTCP_MESSAGE ", 13) == 0) {
 		/* bitchx (and ircii?) sends this */
 		msg += 13;
 		dcc->mirc_ctcp = FALSE;
-	} else if (g_strncasecmp(msg, "CTCP_REPLY ", 11) == 0) {
+	} else if (g_ascii_strncasecmp(msg, "CTCP_REPLY ", 11) == 0) {
 		/* bitchx (and ircii?) sends this */
 		msg += 11;
 		reply = TRUE;
@@ -721,10 +717,9 @@ static void dcc_chat_msg(CHAT_DCC_REC *dcc, const char *msg)
 	ptr = strchr(cmd, ' ');
 	if (ptr != NULL) *ptr++ = '\0'; else ptr = "";
 
-	cmd = g_strdup(cmd);
-	g_strup(cmd);
+	cmd = g_ascii_strup(cmd, -1);
 
-	g_strdown(event+9);
+	ascii_strdown(event+9);
 	if (!signal_emit(event, 2, dcc, ptr)) {
 		signal_emit(reply ? "default dcc reply" :
 			    "default dcc ctcp", 3, dcc, cmd, ptr);
@@ -763,7 +758,7 @@ static void ctcp_reply_dcc_reject(IRC_SERVER_REC *server, const char *data,
 
 	/* default REJECT handler checks args too -
 	   we don't care about it in DCC chats. */
-	if (g_strncasecmp(data, "CHAT", 4) == 0 &&
+	if (g_ascii_strncasecmp(data, "CHAT", 4) == 0 &&
 	    (data[4] == '\0' || data[4] == ' ')) {
 		dcc = dcc_find_request(DCC_CHAT_TYPE, nick, NULL);
 		if (dcc != NULL) dcc_close(dcc);
@@ -782,7 +777,7 @@ static void event_nick(IRC_SERVER_REC *server, const char *data,
 	g_return_if_fail(orignick != NULL);
 
 	params = event_get_params(data, 1, &nick);
-	if (g_strcasecmp(nick, orignick) == 0) {
+	if (g_ascii_strcasecmp(nick, orignick) == 0) {
 		/* shouldn't happen, but just to be sure irssi doesn't
 		   get into infinite loop */
                 g_free(params);
