@@ -30,8 +30,6 @@
 #include "dcc-get.h"
 #include "dcc-send.h"
 
-static int dcc_file_create_mode;
-
 GET_DCC_REC *dcc_get_create(IRC_SERVER_REC *server, CHAT_DCC_REC *chat,
 				   const char *nick, const char *arg)
 {
@@ -57,10 +55,13 @@ static void sig_dcc_destroyed(GET_DCC_REC *dcc)
 char *dcc_get_download_path(const char *fname)
 {
 	char *str, *downpath;
+	char *base;
 
+	base = g_path_get_basename(fname);
 	downpath = convert_home(settings_get_str("dcc_download_path"));
-	str = g_strconcat(downpath, G_DIR_SEPARATOR_S, g_basename(fname), NULL);
+	str = g_strconcat(downpath, G_DIR_SEPARATOR_S, base, NULL);
 	g_free(downpath);
+	g_free(base);
 
 	return str;
 }
@@ -75,7 +76,7 @@ static char *dcc_get_rename_file(const char *fname)
 	newname = g_string_new(NULL);
 	num = 1;
 	do {
-		g_string_sprintf(newname, "%s.%d", fname, num);
+		g_string_printf(newname, "%s.%d", fname, num);
 		num++;
 	} while (stat(newname->str, &statbuf) == 0);
 
@@ -206,6 +207,8 @@ void sig_dccget_connected(GET_DCC_REC *dcc)
 	}
 
 	if (dcc->get_type != DCC_GET_RESUME) {
+		int dcc_file_create_mode = octal2dec(settings_get_int("dcc_file_create_mode"));
+
 		/* we want to overwrite the file, remove it here.
 		   if it gets created after this, we'll fail. */
 		unlink(dcc->file);
@@ -226,7 +229,13 @@ void sig_dccget_connected(GET_DCC_REC *dcc)
 		if (ret != -1) {
 			ret = link(tempfname, dcc->file);
 
-			if (ret == -1 && errno == EPERM) {
+			if (ret == -1 &&
+			    /* Linux */
+			    (errno == EPERM ||
+			     /* FUSE */
+			     errno == ENOSYS ||
+			     /* BSD */
+			     errno == EOPNOTSUPP)) {
 				/* hard links aren't supported - some people
 				   want to download stuff to FAT/NTFS/etc
 				   partitions, so fallback to rename() */
@@ -445,7 +454,7 @@ static void ctcp_msg_dcc_send(IRC_SERVER_REC *server, const char *data,
 		g_memmove(fname, fname+1, len);
                 quoted = TRUE;
 	}
-    
+
 	if (passive && port != 0) {
 		/* This is NOT a DCC SEND request! This is a reply to our
 		   passive request. We MUST check the IDs and then connect to
@@ -492,7 +501,7 @@ static void ctcp_msg_dcc_send(IRC_SERVER_REC *server, const char *data,
 
 	if (passive && port == 0)
 		dcc->pasv_id = p_id; /* Assign the ID to the DCC */
-    
+
 	memcpy(&dcc->addr, &ip, sizeof(ip));
 	if (dcc->addr.family == AF_INET)
 		net_ip2host(&dcc->addr, dcc->addrstr);
@@ -543,7 +552,7 @@ void cmd_dcc_receive(const char *data, DCC_GET_FUNC accept_func,
 		GET_DCC_REC *dcc = tmp->data;
 
 		next = tmp->next;
-		if (IS_DCC_GET(dcc) && g_strcasecmp(dcc->nick, nick) == 0 &&
+		if (IS_DCC_GET(dcc) && g_ascii_strcasecmp(dcc->nick, nick) == 0 &&
 		    (dcc_is_waiting_user(dcc) || dcc->from_dccserver) &&
 		    (*fname == '\0' || strcmp(dcc->arg, fname) == 0)) {
 			found = TRUE;
@@ -566,12 +575,6 @@ static void cmd_dcc_get(const char *data)
 	cmd_dcc_receive(data, dcc_get_connect, dcc_get_passive);
 }
 
-static void read_settings(void)
-{
-	dcc_file_create_mode =
-		octal2dec(settings_get_int("dcc_file_create_mode"));
-}
-
 void dcc_get_init(void)
 {
         dcc_register_type("GET");
@@ -579,10 +582,8 @@ void dcc_get_init(void)
 	settings_add_str("dcc", "dcc_download_path", "~");
 	settings_add_int("dcc", "dcc_file_create_mode", 644);
 
-        read_settings();
 	signal_add("dcc destroyed", (SIGNAL_FUNC) sig_dcc_destroyed);
 	signal_add("ctcp msg dcc send", (SIGNAL_FUNC) ctcp_msg_dcc_send);
-	signal_add("setup changed", (SIGNAL_FUNC) read_settings);
 	command_bind("dcc get", NULL, (SIGNAL_FUNC) cmd_dcc_get);
 }
 
@@ -591,6 +592,5 @@ void dcc_get_deinit(void)
         dcc_unregister_type("GET");
 	signal_remove("dcc destroyed", (SIGNAL_FUNC) sig_dcc_destroyed);
 	signal_remove("ctcp msg dcc send", (SIGNAL_FUNC) ctcp_msg_dcc_send);
-	signal_remove("setup changed", (SIGNAL_FUNC) read_settings);
 	command_unbind("dcc get", (SIGNAL_FUNC) cmd_dcc_get);
 }
