@@ -19,6 +19,7 @@
 */
 
 #define NEED_PERL_H
+#define PERL_NO_GET_CONTEXT
 #include "module.h"
 #include "modules.h"
 #include "signals.h"
@@ -54,7 +55,7 @@ static GSList *use_protocols;
 /* returns the package who called us */
 const char *perl_get_package(void)
 {
-	return SvPV(perl_eval_pv("caller", TRUE), PL_na);
+	return SvPV_nolen(perl_eval_pv("caller", TRUE));
 }
 
 /* Parses the package part from function name */
@@ -81,7 +82,7 @@ SV *perl_func_sv_inc(SV *func, const char *package)
 	if (SvPOK(func)) {
 		/* prefix with package name */
 		name = g_strdup_printf("%s::%s", package,
-				       (char *) SvPV(func, PL_na));
+				       SvPV_nolen(func));
 		func = new_pv(name);
                 g_free(name);
 	} else {
@@ -248,9 +249,9 @@ void irssi_callXS(void (*subaddr)(pTHX_ CV* cv), CV *cv, SV **mark)
 	dSP;
 
 	PUSHMARK(mark);
-	(*subaddr)(aTHX_ cv);
-
 	PUTBACK;
+
+	(*subaddr)(aTHX_ cv);
 }
 
 void perl_chatnet_fill_hash(HV *hv, CHATNET_REC *chatnet)
@@ -301,6 +302,7 @@ void perl_connect_fill_hash(HV *hv, SERVER_CONNECT_REC *conn)
 
 	hv_store(hv, "reconnection", 12, newSViv(conn->reconnection), 0);
 	hv_store(hv, "no_autojoin_channels", 20, newSViv(conn->no_autojoin_channels), 0);
+	hv_store(hv, "no_autosendcmd", 14, newSViv(conn->no_autosendcmd), 0);
 	hv_store(hv, "unix_socket", 11, newSViv(conn->unix_socket), 0);
 	hv_store(hv, "use_ssl", 7, newSViv(conn->use_ssl), 0);
 	hv_store(hv, "no_connect", 10, newSViv(conn->no_connect), 0);
@@ -568,6 +570,7 @@ static void perl_register_protocol(CHAT_PROTOCOL_REC *rec)
 		"Nick"
 	};
 	static char *find_use_code =
+		"use lib qw(%s);\n"
 		"my $pkg = Irssi::%s; $pkg =~ s/::/\\//;\n"
 		"foreach my $i (@INC) {\n"
 		"  return 1 if (-f \"$i/$pkg.pm\");\n"
@@ -626,7 +629,8 @@ static void perl_register_protocol(CHAT_PROTOCOL_REC *rec)
 		perl_eval_pv(code, TRUE);
 	}
 
-	pcode = g_strdup_printf(find_use_code, name);
+	pcode = g_strdup_printf(find_use_code,
+	                        settings_get_str("perl_use_lib"), name);
 	sv = perl_eval_pv(pcode, TRUE);
 	g_free(pcode);
 
