@@ -19,13 +19,13 @@
 */
 
 #define NEED_PERL_H
+#define PERL_NO_GET_CONTEXT
 #include "module.h"
 #include "modules.h"
 #include "core.h"
 #include "signals.h"
 #include "misc.h"
 #include "settings.h"
-#include "lib-config/iconfig.h" /* FIXME: remove before .99 */
 
 #include "perl-core.h"
 #include "perl-common.h"
@@ -35,15 +35,13 @@
 #include "XSUB.h"
 #include "irssi-core.pl.h"
 
-/* For compatibility with perl 5.004 and older */
-#ifndef HAVE_PL_PERL
-#  define PL_perl_destruct_level perl_destruct_level
-#endif
+extern char **environ;
 
 GSList *perl_scripts;
 PerlInterpreter *my_perl;
 
 static int print_script_errors;
+static char *perl_args[] = {"", "-e", "0"};
 
 #define IS_PERL_SCRIPT(file) \
 	(strlen(file) > 3 && strcmp(file+strlen(file)-3, ".pl") == 0)
@@ -61,9 +59,6 @@ static void perl_script_destroy_package(PERL_SCRIPT_REC *script)
 
 	perl_call_pv("Irssi::Core::destroy", G_VOID|G_EVAL|G_DISCARD);
 
-	SPAGAIN;
-
-	PUTBACK;
 	FREETMPS;
 	LEAVE;
 }
@@ -117,7 +112,6 @@ static void xs_init(pTHX)
 /* Initialize perl interpreter */
 void perl_scripts_init(void)
 {
-	char *args[] = {"", "-e", "0"};
 	char *code, *use_code;
 
 	perl_scripts = NULL;
@@ -127,7 +121,7 @@ void perl_scripts_init(void)
 	my_perl = perl_alloc();
 	perl_construct(my_perl);
 
-	perl_parse(my_perl, xs_init, 3, args, NULL);
+	perl_parse(my_perl, xs_init, G_N_ELEMENTS(perl_args), perl_args, NULL);
 #if PERL_STATIC_LIBS == 1
 	perl_eval_pv("Irssi::Core::boot_Irssi_Core();", TRUE);
 #endif
@@ -168,6 +162,7 @@ void perl_scripts_deinit(void)
 	/*perl_eval_pv("eval { foreach my $lib (@DynaLoader::dl_librefs) { DynaLoader::dl_unload_file($lib); } }", TRUE);*/
 
 	/* perl interpreter */
+	PL_perl_destruct_level = 1;
 	perl_destruct(my_perl);
 	perl_free(my_perl);
 	my_perl = NULL;
@@ -193,7 +188,7 @@ static char *script_file_get_name(const char *path)
 {
 	char *name;
 
-        name = g_strdup(g_basename(path));
+        name = g_path_get_basename(path);
 	script_fix_name(name);
         return name;
 }
@@ -207,7 +202,7 @@ static char *script_data_get_name(void)
 	name = g_string_new(NULL);
         n = 1;
 	do {
-		g_string_sprintf(name, "data%d", n);
+		g_string_printf(name, "data%d", n);
                 n++;
 	} while (perl_script_find(name->str) != NULL);
 
@@ -220,8 +215,6 @@ static int perl_script_eval(PERL_SCRIPT_REC *script)
 {
 	dSP;
 	char *error;
-	int retcount;
-	SV *ret;
 
 	ENTER;
 	SAVETMPS;
@@ -232,26 +225,23 @@ static int perl_script_eval(PERL_SCRIPT_REC *script)
 	XPUSHs(sv_2mortal(new_pv(script->name)));
 	PUTBACK;
 
-	retcount = perl_call_pv(script->path != NULL ?
-				"Irssi::Core::eval_file" :
-				"Irssi::Core::eval_data",
-				G_EVAL|G_SCALAR);
+	perl_call_pv(script->path != NULL ?
+                              "Irssi::Core::eval_file" :
+                              "Irssi::Core::eval_data",
+                              G_EVAL|G_DISCARD);
 	SPAGAIN;
 
         error = NULL;
 	if (SvTRUE(ERRSV)) {
-		error = SvPV(ERRSV, PL_na);
+		error = SvPV_nolen(ERRSV);
 
 		if (error != NULL) {
 			error = g_strdup(error);
 			signal_emit("script error", 2, script, error);
 			g_free(error);
 		}
-	} else if (retcount > 0) {
-		ret = POPs;
 	}
 
-	PUTBACK;
 	FREETMPS;
 	LEAVE;
 
@@ -267,7 +257,7 @@ static PERL_SCRIPT_REC *script_load(char *name, const char *path,
 	/* if there's a script with a same name, destroy it */
 	script = perl_script_find(name);
 	if (script != NULL)
-		perl_script_destroy(script);
+		perl_script_unload(script);
 
 	script = g_new0(PERL_SCRIPT_REC, 1);
 	script->name = name;
@@ -445,6 +435,10 @@ static void sig_autorun(void)
 
 void perl_core_init(void)
 {
+	int argc = G_N_ELEMENTS(perl_args);
+	char **argv = perl_args;
+
+	PERL_SYS_INIT3(&argc, &argv, &environ);
         print_script_errors = 1;
 	settings_add_str("perl", "perl_use_lib", PERL_USE_LIB);
 
@@ -470,4 +464,5 @@ void perl_core_deinit(void)
 	perl_signals_deinit();
 
 	signal_remove("script error", (SIGNAL_FUNC) sig_script_error);
+	PERL_SYS_TERM();
 }
