@@ -19,6 +19,7 @@
 */
 
 #define NEED_PERL_H
+#define PERL_NO_GET_CONTEXT
 #include "module.h"
 #include "modules.h"
 #include "signals.h"
@@ -302,7 +303,7 @@ static void perl_call_signal(PERL_SCRIPT_REC *script, SV *func,
 	SPAGAIN;
 
 	if (SvTRUE(ERRSV)) {
-		char *error = g_strdup(SvPV(ERRSV, PL_na));
+		char *error = g_strdup(SvPV_nolen(ERRSV));
 		signal_emit("script error", 2, script, error);
                 g_free(error);
                 rec = NULL;
@@ -323,7 +324,6 @@ static void perl_call_signal(PERL_SCRIPT_REC *script, SV *func,
                         GList **ret = arg;
 			GList *out = NULL;
                         void *val;
-			STRLEN len;
                         int count;
 
 			av = (AV *) SvRV(saved_args[n]);
@@ -331,7 +331,7 @@ static void perl_call_signal(PERL_SCRIPT_REC *script, SV *func,
 			while (count-- >= 0) {
 				sv = av_shift(av);
 				if (SvPOKp(sv))
-					val = g_strdup(SvPV(sv, len));
+					val = g_strdup(SvPV_nolen(sv));
 				else
                                         val = GINT_TO_POINTER(SvIV(sv));
 
@@ -345,7 +345,6 @@ static void perl_call_signal(PERL_SCRIPT_REC *script, SV *func,
 		}
 	}
 
-	PUTBACK;
 	FREETMPS;
 	LEAVE;
 }
@@ -435,7 +434,7 @@ static void perl_signal_remove_list_one(GSList **siglist, PERL_SIGNAL_REC *rec)
 
 #define sv_func_cmp(f1, f2) \
 	(f1 == f2 || (SvPOK(f1) && SvPOK(f2) && \
-		strcmp((char *) SvPV_nolen(f1), (char *) SvPV_nolen(f2)) == 0))
+		strcmp(SvPV_nolen(f1), SvPV_nolen(f2)) == 0))
 
 static void perl_signal_remove_list(GSList **list, SV *func)
 {
@@ -476,7 +475,7 @@ void perl_command_bind_to(const char *cmd, const char *category,
 	g_free(signal);
 }
 
-void perl_command_runsub(const char *cmd, const char *data, 
+void perl_command_runsub(const char *cmd, const char *data,
 			 SERVER_REC *server, WI_ITEM_REC *item)
 {
 	command_runsub(cmd, data, server, item);
